@@ -21,6 +21,8 @@
 
 #define	USE_SHM	// FIXME
 
+#define	EPSILON		(1.0/65536.0)	/* 16bit EPSILON should be good enough */
+
 
 /* --- defines --- */
 #define	SHM_IMAGE_HEIGHT	(32)
@@ -83,11 +85,11 @@ gwm_thumb_nails_set_active (gboolean thumb_nails_enabled)
 
 GwmThumbNail*
 gwm_thumb_nail_new (guint          default_color,
-		    gpointer       user_data,
-		    GDestroyNotify dtor,
 		    guint          width,
 		    guint          height,
-		    glong          grow_request_id)
+		    glong          grow_request_id,
+		    gpointer       user_data,
+		    GDestroyNotify dtor)
 {
   GwmThumbNail *nail;
 
@@ -116,6 +118,7 @@ gwm_thumb_nail_destroy (GwmThumbNail *nail)
 
   g_return_if_fail (nail != NULL);
   g_return_if_fail (nail->pixbuf != NULL);
+  g_return_if_fail (g_slist_find (gwm_thumb_nails, nail) != NULL); /* FIXME: paranoid */
 
   gwm_thumb_nails = g_slist_remove (gwm_thumb_nails, nail);
   dtor = nail->dtor;
@@ -130,6 +133,7 @@ gwm_thumb_nail_destroy (GwmThumbNail *nail)
       g_free (node);
       node = next;
     }
+  nail->size_list = NULL;
   dtor (nail->user_data);
   g_free (nail);
 }
@@ -274,6 +278,7 @@ pixbuf_from_drawable (GdkDrawable *drawable,
   gint sheight = gdk_screen_height ();
   GdkColormap *cmap = gdk_window_get_colormap (drawable);
   GdkPixbuf *shot_pixbuf = NULL;
+  GdkVisual *visual;
 
   /* the drawable relative rectangle (x, y, width, height) must be
    * confined to the root window rectangle (0, 0, swidth, sheight)
@@ -283,10 +288,10 @@ pixbuf_from_drawable (GdkDrawable *drawable,
   if (!shm_image)
     shm_image = gdk_image_new_shared_with_pixmap (NULL, swidth, SHM_IMAGE_HEIGHT, &shm_pixmap);
 
-  /* fallback to root window's colormap */
+  /* fallback to root window's colormap and fetch visual */
   if (!cmap)
     {
-      GdkVisual *visual = gdk_window_get_visual (drawable);
+      visual = gdk_window_get_visual (drawable);
 
       if (visual && visual->depth == gdk_window_get_visual (NULL)->depth)
 	cmap = gdk_window_get_colormap (NULL);
@@ -294,11 +299,17 @@ pixbuf_from_drawable (GdkDrawable *drawable,
       if (!cmap)	/* guess we're screwed */
 	return NULL;
     }
+  else
+    visual = gdk_colormap_get_visual (cmap);
+
+  /* we need a visual for depth determination */
+  if (!visual)
+    return NULL;
 
   /* for depth matches, we first try to grab the image using our global
    * shared memory pixmap image
    */
-  if (shm_image && shm_image->depth == gdk_window_get_visual (drawable)->depth)
+  if (shm_image && shm_image->depth == visual->depth)
     {
       GdkGCValues gc_values;
       GdkGC *gc;
@@ -306,6 +317,8 @@ pixbuf_from_drawable (GdkDrawable *drawable,
 
       gc_values.subwindow_mode = GDK_INCLUDE_INFERIORS;
       gc = gdk_gc_new_with_values (drawable, &gc_values, GDK_GC_SUBWINDOW);
+      if (!gc)
+	return NULL;
 
       shot_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, width, height);
 
@@ -330,7 +343,6 @@ pixbuf_from_drawable (GdkDrawable *drawable,
 	  y += n;
 	  pix_y += n;
 	}
-
       gdk_gc_unref (gc);
     }
 
@@ -349,12 +361,15 @@ pixbuf_from_drawable (GdkDrawable *drawable,
 	  gc_values.subwindow_mode = GDK_INCLUDE_INFERIORS;
 	  gc = gdk_gc_new_with_values (drawable, &gc_values, GDK_GC_SUBWINDOW);
 
-          gdk_error_trap_push ();
-	  gdk_draw_pixmap (pixmap, gc, drawable, x, y, 0, 0, width, height);
-	  gdk_flush ();
-          if (!gdk_error_trap_pop ())
-	    shot_pixbuf = gdk_pixbuf_from_image (image, 0, 0, width, height, cmap);
-	  gdk_gc_unref (gc);
+	  if (gc)
+	    {
+	      gdk_error_trap_push ();
+	      gdk_draw_pixmap (pixmap, gc, drawable, x, y, 0, 0, width, height);
+	      gdk_flush ();
+	      if (!gdk_error_trap_pop ())
+		shot_pixbuf = gdk_pixbuf_from_image (image, 0, 0, width, height, cmap);
+	      gdk_gc_unref (gc);
+	    }
 
 	  gdk_image_destroy (image);
 	  gdk_pixmap_unref (pixmap);
@@ -382,7 +397,7 @@ gwm_thumb_nail_update_drawable (GwmThumbNail *nail,
 				gint	      drawable_x,
 				gint	      drawable_y)
 {
-  GdkPixbuf *pixbuf;
+  GdkPixbuf *pixbuf = NULL;
   gint swidth = gdk_screen_width ();
   gint sheight = gdk_screen_height ();
   gdouble xscale, yscale;
@@ -397,6 +412,8 @@ gwm_thumb_nail_update_drawable (GwmThumbNail *nail,
   gdk_window_get_size (drawable, &xx, &yy);
   drawable_width = xx;
   drawable_height = yy;
+  if (xx < 1 || yy < 1)
+    return FALSE;
 
   /* check out of screen bounds */
   if (drawable_x >= swidth || drawable_y >= sheight ||
@@ -449,10 +466,12 @@ gwm_thumb_nail_update_drawable (GwmThumbNail *nail,
     nail->thumb_row = 0;
 
   /* grab screen portion */
-  pixbuf = pixbuf_from_drawable (drawable,
-				 thumb_x0 * xscale, thumb_y0 * yscale,
-				 (thumb_x1 - thumb_x0) * xscale,
-				 (thumb_y1 - thumb_y0) * yscale);
+  if ((thumb_x1 - thumb_x0) * xscale > 1 + EPSILON &&
+      (thumb_y1 - thumb_y0) * yscale > 1 + EPSILON)
+    pixbuf = pixbuf_from_drawable (drawable,
+				   thumb_x0 * xscale, thumb_y0 * yscale,
+				   (thumb_x1 - thumb_x0) * xscale,
+				   (thumb_y1 - thumb_y0) * yscale);
   if (!pixbuf)
     return FALSE;
   
@@ -516,7 +535,7 @@ gwm_thumb_nail_update_drawable (GwmThumbNail *nail,
 #include <X11/extensions/XShm.h>
 #include <sys/types.h>
 #include <sys/shm.h>
-#endif	USE_SHM
+#endif	/* USE_SHM */
 
 GdkImage*
 gdk_image_new_shared_with_pixmap (GdkWindow  *window,
@@ -621,7 +640,8 @@ gdk_image_get (GdkWindow *window,
 
   win_private = (GdkWindowPrivate *) window;
   visual = gdk_window_get_visual (window);
-  if (win_private->destroyed || width < 1 || height < 1 || !visual)
+  if (win_private->destroyed || width < 1 || height < 1 ||
+      (win_private->window_type != GDK_WINDOW_PIXMAP && !visual))
     return NULL;
 
   private = g_new (GdkImagePrivate, 1);
