@@ -215,7 +215,7 @@ gwmh_init (void)
   static volatile guint n_init_calls = 0;
   XWindowAttributes attribs = { 0, };
   guint32 *property_data;
-  guint size = 0;
+  gint size = 0;
   guint i;
 
   n_init_calls++;
@@ -439,8 +439,8 @@ get_typed_property_data (Display *xdisplay,
 
 	      if (count && tlist && tlist[0])
 		{
-		  mem = g_strdup (tlist[0]);
-		  *size_p = strlen (mem);
+		  mem = (guint8 *)g_strdup (tlist[0]);
+		  *size_p = strlen ((char *)mem);
 		}
 	      if (tlist)
 		gdk_free_text_list (tlist);
@@ -611,7 +611,7 @@ get_task_root_and_frame (GwmhTask *task)
 {
   Window *children = NULL, xframe = 0;
   Window xwin = task->xwin, xparent = xwin, xroot = xwin;
-  int size = 0;
+  guint size = 0;
   
   gdk_error_trap_push ();
 
@@ -646,7 +646,10 @@ get_task_root_and_frame (GwmhTask *task)
     {
       xparent = None;
       xframe = None;
-      g_warning (G_GNUC_PRETTY_FUNCTION "(): task window id %ld invalid?", task->xwin);
+      /* aparently this can happen if a window is destructed shortly after
+       * creation, just be silent.
+       * g_warning (G_GNUC_PRETTY_FUNCTION "(): task window id %ld invalid?", task->xwin);
+       */
     }
 
   if (!task->sroot || GSTC_PARENT_XWINDOW (task->sroot) != xparent)
@@ -842,6 +845,8 @@ gwmh_property_atom2info (Atom     atom,
 {
   static const Atom gwmh_XA_WM_HINTS = XA_WM_HINTS;
   static const Atom gwmh_XA_WM_NAME = XA_WM_NAME;
+  static const Atom gwmh_XA_WM_ICON_NAME = XA_WM_ICON_NAME;
+  
   static const struct {
     const Atom *atom_p;
     guint       iflag;
@@ -865,6 +870,7 @@ gwmh_property_atom2info (Atom     atom,
     { &XA_WM_STATE,			GWMH_TASK_INFO_ICONIFIED, },
     { &gwmh_XA_WM_HINTS,                GWMH_TASK_INFO_WM_HINTS, },
     { &gwmh_XA_WM_NAME,			GWMH_TASK_INFO_MISC, },
+    { &gwmh_XA_WM_ICON_NAME,            GWMH_TASK_INFO_MISC, },
   };
   guint i;
   
@@ -1077,7 +1083,8 @@ gwmh_desk_update (GwmhDeskInfoMask imask)
   
   if (imask & GWMH_DESK_INFO_CLIENT_LIST)
     {
-      gint n_tasks, size = 0;
+      guint n_tasks = 0;
+      gint size = 0;
       guint32 *task_ids;
 
       if (!gwmh_gnome_wm_win)
@@ -1212,6 +1219,23 @@ gwmh_task_update (GwmhTask        *task,
 	  ichanges |= GWMH_TASK_INFO_MISC;
 	}
       g_free (name);
+
+      name = get_typed_property_data (xdisplay, xwindow,
+				      XA_WM_ICON_NAME,
+				      XA_STRING,
+				      &size, 8);
+      if (!name && !size)
+	name = get_typed_property_data (xdisplay, xwindow,
+					XA_WM_ICON_NAME,
+					XA_COMPOUND_TEXT,
+					&size, 8);
+      if (!gwmh_string_equals (task->icon_name, name))
+	{
+	  g_free (task->icon_name);
+	  task->icon_name = g_strdup (name);
+	  ichanges |= GWMH_TASK_INFO_MISC;
+	}
+      g_free (name);
     }
   
   if (imask & GWMH_TASK_INFO_FOCUSED)
@@ -1362,7 +1386,8 @@ gwmh_task_update (GwmhTask        *task,
   if (imask & GWMH_TASK_INFO_ALLOCATION)
     {
       Window dummy_win;
-      guint border, depth, x = 0, y = 0, width = 0, height = 0;
+      gint x = 0, y = 0;
+      guint border, depth, width = 0, height = 0;
       
       if (imask & GWMH_TASK_INFO_FRAME_GEO)
 	{
@@ -1685,6 +1710,7 @@ task_new (GdkWindow *window)
   task = g_new0 (GwmhTask, 1);
   gwmh_desk.client_list = g_list_prepend (gwmh_desk.client_list, task);
   task->name = NULL;
+  task->icon_name = NULL;
   task->frame_x = -1;
   task->frame_y = -1;
   task->win_x = -1;
@@ -1782,6 +1808,8 @@ task_delete (GwmhTask *task)
   gwmh_desk.client_list = g_list_remove (gwmh_desk.client_list, task);
   g_free (task->name);
   task->name = "DELETED";
+  g_free (task->icon_name);
+  task->icon_name = NULL;
   
   if (g_list_find (gwmh_desk.client_list, task))
     g_error ("deleted task still in client_list\n"); /* FIXME */
@@ -1890,6 +1918,21 @@ gwmh_task_get_qdata (GwmhTask *task,
   return g_datalist_id_get_data (&task->datalist, quark);
 }
 
+gpointer
+gwmh_task_steal_qdata (GwmhTask *task,
+		       GQuark    quark)
+{
+  gpointer data;
+
+  g_return_val_if_fail (task != NULL, NULL);
+  
+  data = g_datalist_id_get_data (&task->datalist, quark);
+  if (data)
+    g_datalist_id_remove_no_notify (&task->datalist, quark);
+
+  return data;
+}
+
 void
 gwmh_task_kill (GwmhTask *task)
 {
@@ -2135,7 +2178,8 @@ gwmh_task_get_mini_icon (GwmhTask   *task,
   guint32 *atomdata;
   Window xwindow = task->xwin;
   Window root;
-  int x, y, b, width, height, depth;
+  int x, y;
+  guint b, width, height, depth;
   gint size;
   Display *xdisplay;
 
