@@ -18,7 +18,8 @@ static gchar *contributors[] = {
 	"Shawn T. Amundson",
 	"Erik Andersen",
 	"Jon Anhold",
-	"Shooby Ban",
+	/* for languages that can't display aacute, replace '' with 'a' */
+	N_("Szabolcs 'Shooby' Bn"),
 	"Martin Baulig",
 	"Tom Bech",
 	"Andreas Beck",
