@@ -18,6 +18,7 @@
 #define HAVE_PANEL_PIXEL_SIZE 1
 #define HAVE_PANEL_DRAW_SIGNAL 1
 #define HAVE_APPLET_QUEUE_RESIZE 1
+#define HAVE_APPLET_SIZE_ULTRA_TINY_AND_RIDICULOUS 1
 
 BEGIN_GNOME_DECLS
 
@@ -28,11 +29,13 @@ typedef GNOME_Panel_OrientType PanelOrientType;
 #define ORIENT_RIGHT GNOME_Panel_ORIENT_RIGHT
 
 enum {
+	PIXEL_SIZE_ULTRA_TINY = 12,
 	PIXEL_SIZE_TINY = 24,
 	PIXEL_SIZE_SMALL = 36,
 	PIXEL_SIZE_STANDARD = 48,
 	PIXEL_SIZE_LARGE = 64,
-	PIXEL_SIZE_HUGE = 80
+	PIXEL_SIZE_HUGE = 80,
+	PIXEL_SIZE_RIDICULOUS = 128
 };
 
 typedef GNOME_Panel_BackType PanelBackType;
@@ -41,6 +44,7 @@ typedef GNOME_Panel_BackType PanelBackType;
 #define PANEL_BACK_PIXMAP GNOME_Panel_BACK_PIXMAP
 
 
+#define TYPE_APPLET_WIDGET          (applet_widget_get_type ())
 #define APPLET_WIDGET(obj)          GTK_CHECK_CAST (obj, applet_widget_get_type (), AppletWidget)
 #define APPLET_WIDGET_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, applet_widget_get_type (), AppletWidgetClass)
 #define IS_APPLET_WIDGET(obj)       GTK_CHECK_TYPE (obj, applet_widget_get_type ())
@@ -121,7 +125,7 @@ typedef GtkWidget *(*AppletFactoryActivator)(const char *goad_id, const char **p
 /* Returns TRUE if the factory can activate this applet */
 typedef gboolean (*AppletFactoryQuerier)(const char *goad_id);
 
-guint		applet_widget_get_type		(void);
+guint		applet_widget_get_type		(void) G_GNUC_CONST;
 
 void            applet_factory_new(const char *goad_id,
 				   AppletFactoryQuerier qfunc,
