@@ -39,43 +39,51 @@ extern GlobalConfig global_config;
 extern PanelWidget *current_panel;
 
 static void
-move_applet_callback(GtkWidget *widget, AppletInfo *info)
+move_applet_callback (GtkWidget *widget, AppletInfo *info)
 {
-	PanelWidget    *panel;
+	PanelWidget *panel;
 
-	g_return_if_fail(info != NULL);
+	g_return_if_fail (info != NULL);
+	g_return_if_fail (info->widget != NULL);
+	g_return_if_fail (info->widget->parent != NULL);
+	g_return_if_fail (IS_PANEL_WIDGET (info->widget->parent));
 
-	panel = PANEL_WIDGET(info->widget->parent);
-	g_return_if_fail(panel!=NULL);
+	panel = PANEL_WIDGET (info->widget->parent);
 
-	panel_widget_applet_drag_start(panel, info->widget);
+	panel_widget_applet_drag_start (panel, info->widget,
+					PW_DRAG_OFF_CENTER);
 }
 
 /*destroy widgets and call the above cleanup function*/
 void
-panel_clean_applet(AppletInfo *info)
+panel_clean_applet (AppletInfo *info)
 {
-	g_return_if_fail(info != NULL);
+	g_return_if_fail (info != NULL);
 
-	if(info->widget) {
+	if (info->widget != NULL) {
 		if(info->type == APPLET_STATUS) {
-			status_applet_put_offscreen(info->data);
+			status_applet_put_offscreen (info->data);
 		}
 		/* destroy will remove it from the panel */
-		gtk_widget_destroy(info->widget);
+		gtk_widget_destroy (info->widget);
 		info->widget = NULL;
 	}
+
+	info->data = NULL;
 }
 
 static gboolean
 kill_applet_in_idle(gpointer data)
 {
 	AppletInfo *info = data;
- 	if (info->type == APPLET_EXTERN) {
- 		Extern *ext = info->data;
- 		ext->clean_remove = TRUE;
- 	}
-	panel_clean_applet(info);
+	if (info->type == APPLET_EXTERN) {
+		Extern *ext = info->data;
+		ext->clean_remove = TRUE;
+	} else if (info->type == APPLET_SWALLOW) {
+		Swallow *swallow = info->data;
+		swallow->clean_remove = TRUE;
+	}
+	panel_clean_applet (info);
 	return FALSE;
 }
 
@@ -111,7 +119,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		if(strcmp(menu->name,"properties")==0)
 			launcher_properties(menu->info->data);
 		else if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "launchers.html");
+			panel_show_help ("launchers.html");
 		break;
 	case APPLET_DRAWER: 
 		if(strcmp(menu->name,"properties")==0) {
@@ -119,11 +127,11 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			g_assert(drawer);
 			panel_config(drawer->drawer);
 		} else if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "drawers.html");
+			panel_show_help ("drawers.html");
 		break;
 	case APPLET_SWALLOW:
  		if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "specialobjects.html#SWALLOWEDAPP");
+			panel_show_help ("specialobjects.html#SWALLOWEDAPP");
 #if 0
 		if(strcmp(menu->name,"properties")==0) {
 			Swallow *swallow = info->data;
@@ -137,7 +145,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			menu_properties(menu->info->data);
 		else if(strcmp(menu->name,"edit_menus")==0) {
 			char *tmp;
-			if((tmp = gnome_is_program_in_path("gmenu")))  {
+			if((tmp = panel_is_program_in_path("gmenu")))  {
 				gnome_execute_async(NULL, 1, &tmp);
 				g_free(tmp);
 			}
@@ -146,7 +154,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			char *page;
 			page = (menu2->path && strcmp (menu2->path,"."))
 				? "menus.html" : "mainmenu.html";
-			panel_pbox_help_cb (NULL, 0, page);
+			panel_show_help (page);
 		}
 		break;
 	case APPLET_LOCK: {
@@ -164,7 +172,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		char *command = NULL;
 		gboolean freeit = FALSE;
 		if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "specialobjects.html#LOCKBUTTON");
+			panel_show_help ("specialobjects.html#LOCKBUTTON");
 		else if (!strcmp (menu->name, "restart")) {
 			command = "xscreensaver-command -exit ; xscreensaver &";
 		} else if (!strcmp (menu->name, "prefs")) {
@@ -182,15 +190,15 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 	}
 	case APPLET_LOGOUT:
 		if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "specialobjects.html#LOGOUTBUTTON");
+			panel_show_help ("specialobjects.html#LOGOUTBUTTON");
 		break;
 	case APPLET_STATUS:
 		if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "specialobjects.html#STATUSDOC");
+			panel_show_help ("specialobjects.html#STATUSDOC");
 		break;
 	case APPLET_RUN:
 		if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "specialobjects.html#RUNBUTTON");
+			panel_show_help ("specialobjects.html#RUNBUTTON");
 		break;
 	default: break;
 	}
@@ -206,15 +214,17 @@ applet_menu_deactivate(GtkWidget *w, AppletInfo *info)
 		BASEP_WIDGET(panel)->autohide_inhibit = FALSE;
 }
 
-static AppletUserMenu *
-applet_get_callback(GList *user_menu, const char *name)
+AppletUserMenu *
+applet_get_callback (GList *user_menu, const char *name)
 {
-	GList *list;
-	for(list=user_menu;list!=NULL;list=g_list_next(list)) {
-		AppletUserMenu *menu = list->data;
-		if(strcmp(menu->name,name)==0)
+	GList *li;
+
+	for (li = user_menu; li != NULL; li = li->next) {
+		AppletUserMenu *menu = li->data;
+		if (strcmp (menu->name, name) == 0)
 			return menu;
 	}
+
 	return NULL;	
 }
 
@@ -226,25 +236,26 @@ applet_add_callback(AppletInfo *info,
 {
 	AppletUserMenu *menu;
 
-	g_return_if_fail(info != NULL);
+	g_return_if_fail (info != NULL);
 	
-	if((menu=applet_get_callback(info->user_menu,callback_name))==NULL) {
-		menu = g_new0(AppletUserMenu,1);
-		menu->name = g_strdup(callback_name);
-		menu->stock_item = g_strdup(stock_item);
-		menu->text = g_strdup(menuitem_text);
+	menu = applet_get_callback (info->user_menu, callback_name);
+	if (menu == NULL) {
+		menu = g_new0 (AppletUserMenu, 1);
+		menu->name = g_strdup (callback_name);
+		menu->stock_item = g_strdup (stock_item);
+		menu->text = g_strdup (menuitem_text);
 		menu->sensitive = TRUE;
 		menu->info = info;
 		menu->menuitem = NULL;
 		menu->submenu = NULL;
-		info->user_menu = g_list_append(info->user_menu,menu);
+		info->user_menu = g_list_append (info->user_menu, menu);
 	} else {
-		if(menu->stock_item)
-			g_free(menu->stock_item);
-		if(menu->text)
-			g_free(menu->text);
-		menu->text = g_strdup(menuitem_text);
-		menu->stock_item = g_strdup(stock_item);
+		g_free (menu->stock_item);
+		menu->stock_item = NULL;
+		g_free (menu->text);
+		menu->text = NULL;
+		menu->text = g_strdup (menuitem_text);
+		menu->stock_item = g_strdup (stock_item);
 	}
 
 	/*make sure the menu is rebuilt*/
@@ -266,29 +277,30 @@ applet_remove_callback(AppletInfo *info, const char *callback_name)
 {
 	AppletUserMenu *menu;
 
-	g_return_if_fail(info != NULL);
+	g_return_if_fail (info != NULL);
 	
-	if((menu=applet_get_callback(info->user_menu,callback_name))!=NULL) {
-		info->user_menu = g_list_remove(info->user_menu,menu);
-		if(menu->name)
-			g_free(menu->name);
-		if(menu->stock_item)
-			g_free(menu->stock_item);
-		if(menu->text)
-			g_free(menu->text);
+	menu = applet_get_callback (info->user_menu, callback_name);
+	if (menu != NULL) {
+		info->user_menu = g_list_remove (info->user_menu, menu);
+		g_free(menu->name);
+		menu->name = NULL;
+		g_free(menu->stock_item);
+		menu->stock_item = NULL;
+		g_free(menu->text);
+		menu->text = NULL;
 		g_free(menu);
 	} else
 		return; /*it just isn't there*/
 
 	/*make sure the menu is rebuilt*/
-	if(info->menu) {
+	if (info->menu != NULL) {
 		GList *list;
-		for(list=info->user_menu;list!=NULL;list=g_list_next(list)) {
+		for (list = info->user_menu; list != NULL; list = list->next) {
 			AppletUserMenu *menu = list->data;
-			menu->menuitem=NULL;
-			menu->submenu=NULL;
+			menu->menuitem = NULL;
+			menu->submenu = NULL;
 		}
-		gtk_widget_unref(info->menu);
+		gtk_widget_unref (info->menu);
 		info->menu = NULL;
 		info->menu_age = 0;
 	}
@@ -301,7 +313,8 @@ applet_callback_set_sensitive(AppletInfo *info, const char *callback_name, int s
 
 	g_return_if_fail(info != NULL);
 	
-	if((menu=applet_get_callback(info->user_menu,callback_name))!=NULL)
+	menu = applet_get_callback(info->user_menu, callback_name);
+	if (menu != NULL)
 		menu->sensitive = sensitive;
 	else
 		return; /*it just isn't there*/
@@ -368,22 +381,22 @@ setup_an_item(AppletUserMenu *menu,
 }
 
 static void
-add_to_submenus(AppletInfo *info,
-		char *path,
-		char *name,
-	       	AppletUserMenu *menu,
-	       	GtkWidget *submenu,
-		GList *user_menu)
+add_to_submenus (AppletInfo *info,
+		 const char *path,
+		 const char *name,
+		 AppletUserMenu *menu,
+		 GtkWidget *submenu,
+		 GList *user_menu)
 {
-	char *n = g_strdup(name);
-	char *p = strchr(n,'/');
+	char *n = g_strdup (name);
+	char *p = strchr (n, '/');
 	char *t;
 	AppletUserMenu *s_menu;
 
 	/*this is the last one*/
-	if(p==NULL) {
-		g_free(n);
-		setup_an_item(menu,submenu,FALSE);
+	if (p == NULL) {
+		g_free (n);
+		setup_an_item (menu, submenu, FALSE);
 		return;
 	}
 	
@@ -397,50 +410,49 @@ add_to_submenus(AppletInfo *info,
 	*p = '\0';
 	p++;
 	
-	t = g_strconcat(path,n,"/",NULL);
-	s_menu = applet_get_callback(user_menu,t);
+	t = g_strconcat (path, n, "/", NULL);
+	s_menu = applet_get_callback (user_menu, t);
 	/*the user did not give us this sub menu, whoops, will create an empty
 	  one then*/
-	if(!s_menu) {
-		s_menu = g_new0(AppletUserMenu,1);
-		s_menu->name = g_strdup(t);
+	if (s_menu == NULL) {
+		s_menu = g_new0 (AppletUserMenu,1);
+		s_menu->name = g_strdup (t);
 		s_menu->stock_item = NULL;
-		s_menu->text = g_strdup(_("???"));
+		s_menu->text = g_strdup (_("???"));
 		s_menu->sensitive = TRUE;
 		s_menu->info = info;
 		s_menu->menuitem = NULL;
 		s_menu->submenu = NULL;
-		info->user_menu = g_list_append(info->user_menu,s_menu);
+		info->user_menu = g_list_append (info->user_menu,s_menu);
 		user_menu = info->user_menu;
-
 	}
 	
-	if(!s_menu->submenu) {
+	if (s_menu->submenu == NULL) {
 		s_menu->submenu = scroll_menu_new();
 		/*a more elegant way to do this should be done
 		  when I don't want to go to sleep */
-		if (s_menu->menuitem) {
+		if (s_menu->menuitem != NULL) {
 			gtk_widget_destroy (s_menu->menuitem);
 			s_menu->menuitem = NULL;
 		}
 	}
-	if(!s_menu->menuitem)
-		setup_an_item(s_menu,submenu,TRUE);
+	if (s_menu->menuitem == NULL)
+		setup_an_item (s_menu, submenu, TRUE);
 	
-	add_to_submenus(info,t,p,menu,s_menu->submenu,user_menu);
+	add_to_submenus (info, t, p, menu, s_menu->submenu, user_menu);
 	
 	g_free(t);
 	g_free(n);
 }
 
 void
-create_applet_menu(AppletInfo *info, gboolean is_basep)
+create_applet_menu (AppletInfo *info, gboolean is_basep)
 {
 	GtkWidget *menuitem, *panel_menu;
 	GList *user_menu = info->user_menu;
 	gchar *pixmap;
 
-	info->menu = scroll_menu_new();
+	info->menu = scroll_menu_new ();
 
 	menuitem = gtk_menu_item_new();
 	setup_menuitem(menuitem,
@@ -463,7 +475,7 @@ create_applet_menu(AppletInfo *info, gboolean is_basep)
 	menuitem = gtk_menu_item_new ();
 
 	pixmap = gnome_pixmap_file ("gnome-panel.png");
-	if (!pixmap) {
+	if (pixmap == NULL) {
 		g_message (_("Cannot find pixmap file %s"), "gnome-panel.png");
 		setup_menuitem (menuitem, NULL, _("Panel"));
 	} else {
@@ -486,15 +498,15 @@ create_applet_menu(AppletInfo *info, gboolean is_basep)
 
 	for(;user_menu!=NULL;user_menu = g_list_next(user_menu)) {
 		AppletUserMenu *menu=user_menu->data;
-		add_to_submenus(info,"",menu->name,menu,info->menu,
-				info->user_menu);
+		add_to_submenus (info, "", menu->name, menu, info->menu,
+				 info->user_menu);
 	}
 
 	/*connect the deactivate signal, so that we can "re-allow" autohide
 	  when the menu is deactivated*/
-	gtk_signal_connect(GTK_OBJECT(info->menu), "deactivate",
-			   GTK_SIGNAL_FUNC(applet_menu_deactivate),
-			   info);
+	gtk_signal_connect (GTK_OBJECT (info->menu), "deactivate",
+			    GTK_SIGNAL_FUNC (applet_menu_deactivate),
+			    info);
 }
 
 static void
@@ -502,16 +514,16 @@ set_data(GtkWidget *menu, gpointer panel)
 {
 	GList *children, *li;
 
-	gtk_object_set_data(GTK_OBJECT(menu), "menu_panel", panel);
+	gtk_object_set_data (GTK_OBJECT (menu), "menu_panel", panel);
 
-	children = gtk_container_children(GTK_CONTAINER(menu));
+	children = gtk_container_children (GTK_CONTAINER (menu));
 
 	for(li = children; li; li = li->next) {
 		GtkMenuItem *item = li->data;
-		if(item->submenu)
-			set_data(item->submenu, panel);
+		if (item->submenu != NULL)
+			set_data (item->submenu, panel);
 	}
-	g_list_free(children);
+	g_list_free (children);
 }
 
 void
@@ -523,8 +535,9 @@ show_applet_menu(AppletInfo *info, GdkEventButton *event)
 
 	panel = get_panel_parent(info->widget);
 
-	if (!info->menu)
+	if (info->menu == NULL)
 		create_applet_menu (info, IS_BASEP_WIDGET (panel));
+	g_assert (info->menu != NULL);
 
 	if(IS_BASEP_WIDGET(panel)) {
 		BASEP_WIDGET(panel)->autohide_inhibit = TRUE;
@@ -532,11 +545,17 @@ show_applet_menu(AppletInfo *info, GdkEventButton *event)
 	}
 	info->menu_age = 0;
 
-	set_data(info->menu, info->widget->parent);
+	set_data (info->menu, info->widget->parent);
 
-	gtk_menu_popup(GTK_MENU(info->menu), NULL, NULL,
-		       global_config.off_panel_popups?applet_menu_position:NULL,
-		       info, event->button, event->time);
+	gtk_menu_popup (GTK_MENU (info->menu),
+			NULL,
+			NULL,
+			global_config.off_panel_popups ?
+			  applet_menu_position :
+			  NULL,
+			info,
+			event->button,
+			event->time);
 }
 
 
@@ -544,15 +563,16 @@ show_applet_menu(AppletInfo *info, GdkEventButton *event)
 static gboolean
 applet_button_press (GtkWidget *widget, GdkEventButton *event, AppletInfo *info)
 {
-	if(event->button==3) {
-		if(!panel_applet_in_drag) {
+	if (event->button == 3) {
+		if ( ! panel_applet_in_drag) {
 			if(info->type == APPLET_SWALLOW) {
 				Swallow *swallow = info->data;
 				GtkWidget *handle_box = swallow->handle_box;
 				if(!GTK_HANDLE_BOX(handle_box)->child_detached)
 					show_applet_menu(info, event);
-			} else
-				show_applet_menu(info, event);
+			} else {
+				show_applet_menu (info, event);
+			}
 		}
 	}
 	/* don't let any button click events to the panel or moving the applets
@@ -560,6 +580,26 @@ applet_button_press (GtkWidget *widget, GdkEventButton *event, AppletInfo *info)
 	return TRUE;
 }
 
+static GList *launchers_to_kill = NULL;
+
+void
+remove_unused_launchers (void)
+{
+	GList *li;
+
+	for (li = launchers_to_kill; li != NULL; li = li->next) {
+		char *file = li->data;
+		li->data = NULL;
+
+		unlink (file);
+
+		g_free (file);
+	}
+
+	g_list_free (launchers_to_kill);
+	launchers_to_kill = NULL;
+}
+
 static void
 applet_destroy (GtkWidget *w, AppletInfo *info)
 {
@@ -569,7 +609,7 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 
 	info->widget = NULL;
 
-	if(info->type == APPLET_DRAWER) {
+	if (info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
 		g_assert(drawer);
 		if(drawer->drawer) {
@@ -578,14 +618,23 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 			PANEL_WIDGET(BASEP_WIDGET(dw)->panel)->master_widget = NULL;
 			gtk_widget_destroy(dw);
 		}
+	} else if (info->type == APPLET_LAUNCHER) {
+		Launcher *launcher = info->data;
 
-		/* HACK!, there is a cleaner solution in the 1.4 version, but
-		 * This is a minimal intrusive change for the stable branch, basically
-		 * the drawer's destroy handler has already run, and we need to free
-		 * the drawer structure here, and not there */
-		g_free (drawer);
+		/* we CAN'T unlink the file here as we may just
+		 * be killing stuff before exit, basically we
+		 * just want to schedule removals until session
+		 * saving */
+
+		if (launcher->dentry->location != NULL) {
+			char *file = g_strdup (launcher->dentry->location);
+			launchers_to_kill = 
+				g_list_prepend (launchers_to_kill, file);
+		}
 	}
-	if(info->menu) {
+
+
+	if (info->menu != NULL) {
 		gtk_widget_unref(info->menu);
 		info->menu = NULL;
 		info->menu_age = 0;
@@ -593,79 +642,81 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 
 	info->type = APPLET_EMPTY;
 
-	info->data=NULL;
+	if (info->data_destroy)
+		info->data_destroy (info->data);
+	info->data_destroy = NULL;
+	info->data = NULL;
 
 	/*free the user menu*/
 	for(li = info->user_menu; li != NULL; li = g_list_next(li)) {
 		AppletUserMenu *umenu = li->data;
-		if(umenu->name)
-			g_free(umenu->name);
-		if(umenu->stock_item)
-			g_free(umenu->stock_item);
-		if(umenu->text)
-			g_free(umenu->text);
+
+		li->data = NULL;
+
+		g_free(umenu->name);
+		umenu->name = NULL;
+		g_free(umenu->stock_item);
+		umenu->stock_item = NULL;
+		g_free(umenu->text);
+		umenu->text = NULL;
+
 		g_free(umenu);
 	}
-	g_list_free(info->user_menu);
+	g_list_free (info->user_menu);
+	info->user_menu = NULL;
 }
 
 gboolean
-register_toy(GtkWidget *applet,
-	     gpointer data,
-	     PanelWidget *panel,
-	     int pos,
-	     gboolean exactpos,
-	     AppletType type)
+register_toy (GtkWidget *applet,
+	      gpointer data,
+	      GDestroyNotify data_destroy,
+	      PanelWidget *panel,
+	      int pos,
+	      gboolean exactpos,
+	      AppletType type)
 {
 	AppletInfo *info;
 	int newpos;
 	gboolean insert_at_pos;
 	
-	g_return_val_if_fail(applet != NULL, FALSE);
-	g_return_val_if_fail(panel != NULL, FALSE);
+	g_return_val_if_fail (applet != NULL, FALSE);
+	g_return_val_if_fail (panel != NULL, FALSE);
 
-	if(!GTK_WIDGET_NO_WINDOW(applet))
-		gtk_widget_set_events(applet, (gtk_widget_get_events(applet) |
-					       APPLET_EVENT_MASK) &
-				      ~( GDK_POINTER_MOTION_MASK |
-					 GDK_POINTER_MOTION_HINT_MASK));
+	if ( ! GTK_WIDGET_NO_WINDOW (applet))
+		gtk_widget_set_events (applet, (gtk_widget_get_events (applet) |
+						APPLET_EVENT_MASK) &
+				       ~( GDK_POINTER_MOTION_MASK |
+					  GDK_POINTER_MOTION_HINT_MASK));
 
-	info = g_new0(AppletInfo,1);
+	info = g_new0 (AppletInfo, 1);
 	info->applet_id = applet_count;
 	info->type = type;
 	info->widget = applet;
 	info->menu = NULL;
 	info->menu_age = 0;
 	info->data = data;
+	info->data_destroy = data_destroy;
 	info->user_menu = NULL;
 
-	/* Since we will be taking care of refcounting by ourselves,
-	 * sink the object.
-	 */
+	gtk_object_set_data (GTK_OBJECT (applet), "applet_info", info);
 
-	/*
-	gtk_widget_ref (info->widget);
-	gtk_object_sink (GTK_OBJECT (info->widget));
-	*/
-
-	gtk_object_set_data(GTK_OBJECT(applet),"applet_info",info);
-
-	if(type == APPLET_DRAWER) {
+	if (type == APPLET_DRAWER) {
 		Drawer *drawer = data;
 		PanelWidget *assoc_panel =
-			PANEL_WIDGET(BASEP_WIDGET(drawer->drawer)->panel);
+			PANEL_WIDGET (BASEP_WIDGET (drawer->drawer)->panel);
 
-		gtk_object_set_data(GTK_OBJECT(applet),
-				    PANEL_APPLET_ASSOC_PANEL_KEY, assoc_panel);
+		gtk_object_set_data (GTK_OBJECT (applet),
+				     PANEL_APPLET_ASSOC_PANEL_KEY, assoc_panel);
 		assoc_panel->master_widget = applet;
 	}
-	gtk_object_set_data(GTK_OBJECT(applet),
-			    PANEL_APPLET_FORBIDDEN_PANELS, NULL);
+
+	gtk_object_set_data (GTK_OBJECT (applet),
+			     PANEL_APPLET_FORBIDDEN_PANELS, NULL);
 		
-	if(!applets) {
-		applets_last = applets = g_slist_append(NULL, info);
+	if (applets == NULL) {
+		applets_last = applets = g_slist_append (NULL, info);
 	} else {
-		applets_last = g_slist_append(applets_last, info);
+		applets_last = g_slist_append (applets_last, info);
 		applets_last = applets_last->next;
 	}
 	applet_count++;
@@ -674,7 +725,7 @@ register_toy(GtkWidget *applet,
 	applets_to_sync = TRUE;
 
 	/*add at the beginning if pos == -1*/
-	if(pos>=0) {
+	if (pos >= 0) {
 		newpos = pos;
 		insert_at_pos = FALSE;
 	} else {
@@ -682,15 +733,15 @@ register_toy(GtkWidget *applet,
 		insert_at_pos = TRUE;
 	}
 	/* if exact pos is on then insert at that precise location */
-	if(exactpos)
+	if (exactpos)
 		insert_at_pos = TRUE;
 
-	if(panel_widget_add_full(panel, applet, newpos, TRUE,
-				 insert_at_pos)==-1) {
+	if (panel_widget_add_full (panel, applet, newpos, TRUE,
+				   insert_at_pos)==-1) {
 		GSList *list;
 		for(list = panels; list != NULL; list = g_slist_next(list))
-			if(panel_widget_add_full(panel, applet, 0,
-						 TRUE, TRUE)!=-1)
+			if (panel_widget_add_full (panel, applet, 0,
+						   TRUE, TRUE)!=-1)
 				break;
 		if(!list) {
 			/*can't put it anywhere, clean up*/
