@@ -25,11 +25,13 @@
 #include "drawer-widget.h"
 #include "gnome-run.h"
 #include "scroll-menu.h"
+#include "gwmh.h"
+#include "tasklist_icon.h"
 
 #define SMALL_ICON_SIZE 20
 
 extern GlobalConfig global_config;
-extern GList *panel_list;
+extern GSList *panel_list;
 
 extern GtkTooltips *panel_tooltips;
 
@@ -39,12 +41,20 @@ static void foobar_widget_realize	(GtkWidget		*w);
 static void foobar_widget_destroy	(GtkObject		*o);
 static void foobar_widget_size_allocate	(GtkWidget		*w,
 					 GtkAllocation		*alloc);
+static gboolean foobar_leave_notify	(GtkWidget *widget,
+					 GdkEventCrossing *event);
+static gboolean foobar_enter_notify	(GtkWidget *widget,
+					 GdkEventCrossing *event);
+static void append_task_menu (FoobarWidget *foo, GtkMenuBar *menu_bar);
+static void setup_task_menu (FoobarWidget *foo);
 
 static GtkWidget *das_global_foobar = NULL;
 static GtkWidget *clock_ebox = NULL;
 
 static GtkWindowClass *parent_class = NULL;
 
+
+
 GtkType
 foobar_widget_get_type (void)
 {
@@ -81,6 +91,8 @@ foobar_widget_class_init (FoobarWidgetClass *klass)
 
 	widget_class->realize = foobar_widget_realize;
 	widget_class->size_allocate = foobar_widget_size_allocate;
+	widget_class->enter_notify_event = foobar_enter_notify;
+	widget_class->leave_notify_event = foobar_leave_notify;
 }
 
 static GtkWidget *
@@ -125,6 +137,32 @@ add_tearoff (GtkMenu *menu)
 	gtk_menu_prepend (menu, item);
 }
 
+static gboolean
+foobar_leave_notify (GtkWidget *widget,
+		     GdkEventCrossing *event)
+{
+	if (GTK_WIDGET_CLASS (parent_class)->leave_notify_event)
+		GTK_WIDGET_CLASS (parent_class)->leave_notify_event (widget,
+								     event);
+
+	return FALSE;
+}
+
+static gboolean
+foobar_enter_notify (GtkWidget *widget,
+		     GdkEventCrossing *event)
+{
+	if (GTK_WIDGET_CLASS (parent_class)->enter_notify_event)
+		GTK_WIDGET_CLASS (parent_class)->enter_notify_event (widget,
+								     event);
+
+	if (global_config.autoraise)
+		gdk_window_raise (widget->window);
+
+	return FALSE;
+}
+
+#if 0
 static void
 url_show (GtkWidget *w, const char *url)
 {
@@ -149,6 +187,7 @@ about_cb (GtkWidget *w, gpointer data)
 	char *v[2] = { "gnome-about" };
 	gnome_execute_async (NULL, 1, v);
 }
+#endif
 
 static void
 gmc_client (GtkWidget *w, gpointer data)
@@ -171,6 +210,7 @@ gnomecal_client (GtkWidget *w, gpointer data)
 				     "It is in the gnome-pim package."));
 }
 
+#if 0
 static GtkWidget *
 append_gnome_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 {
@@ -211,6 +251,7 @@ append_gnome_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
 	return item;
 }
+#endif
 
 static void
 append_gmc_item (GtkWidget *menu, const char *label, char *flag)
@@ -257,7 +298,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 
 	add_menu_separator (menu);
 	
-	char_tmp = gnome_is_program_in_path ("xscreensaver");
+	char_tmp = panel_is_program_in_path ("xscreensaver");
 	if (char_tmp) {	
 		item = pixmap_menu_item_new (_("Lock Screen"), 
 					       "gnome-lockscreen.png");
@@ -298,7 +339,8 @@ append_folder_menu (GtkWidget *menu_bar, const char *label,
 		return NULL;
 	}
 
-	menu = create_fake_menu_at (real_path, FALSE, label, NULL, FALSE);
+	menu = create_fake_menu_at (real_path, FALSE, FALSE,
+				    label, NULL, FALSE);
 	g_free (real_path);
 
 	if (menu == NULL) {
@@ -386,10 +428,11 @@ timeout_cb (gpointer data)
 static void
 set_fooclock_format (GtkWidget *w, char *format)
 {
-	if (!IS_FOOBAR_WIDGET (das_global_foobar))
+	if ( ! IS_FOOBAR_WIDGET (das_global_foobar))
 		return;
 
-	foobar_widget_set_clock_format (FOOBAR_WIDGET (das_global_foobar), _(format));
+	foobar_widget_set_clock_format (FOOBAR_WIDGET (das_global_foobar),
+					_(format));
 }
 
 static void
@@ -431,8 +474,8 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	const char *formats[] = {
 		N_("%H:%M"),
 		N_("%H:%M:%S"),
-		N_("%I:%M %p"),
-		N_("%I:%M:%S %p"),
+		N_("%l:%M %p"),
+		N_("%l:%M:%S %p"),
 		NULL
 	};
 
@@ -475,6 +518,17 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	return item;
 }
 
+void
+foobar_widget_global_set_clock_format (const char *format)
+{
+	if (das_global_foobar == NULL ||
+	    ! IS_FOOBAR_WIDGET (das_global_foobar))
+		return;
+
+	foobar_widget_set_clock_format (FOOBAR_WIDGET (das_global_foobar),
+					format);
+}
+
 void
 foobar_widget_set_clock_format (FoobarWidget *foo, const char *clock_format)
 {
@@ -488,8 +542,9 @@ void
 foobar_widget_update_winhints (FoobarWidget *foo)
 {
 	GtkWidget *w = GTK_WIDGET (foo);
+	GnomeWinLayer layer;
 
-	if (!foo->compliant_wm)
+	if ( ! foo->compliant_wm)
 		return;
 
 	gdk_window_set_hints (w->window, 0, 0, 
@@ -502,19 +557,33 @@ foobar_widget_update_winhints (FoobarWidget *foo)
 	
 	gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS |
 				   WIN_HINTS_DO_NOT_COVER);	
-	gnome_win_hints_set_layer (w, global_config.keep_bottom
-				   ? WIN_LAYER_BELOW
-				   : WIN_LAYER_DOCK);
+	if (global_config.normal_layer) {
+		layer = WIN_LAYER_NORMAL;
+	} else if (global_config.keep_bottom) {
+		layer = WIN_LAYER_BELOW;
+	} else {
+		layer = WIN_LAYER_DOCK;
+	}
+	gnome_win_hints_set_layer (w, layer);
 }
 
 static void
 foobar_widget_realize (GtkWidget *w)
 {
+	gtk_window_set_wmclass (GTK_WINDOW (w),
+				"panel_window", "Panel");
+
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		GTK_WIDGET_CLASS (parent_class)->realize (w);
 
 	foobar_widget_update_winhints (FOOBAR_WIDGET (w));
-	xstuff_set_no_group (w->window);
+	xstuff_set_no_group_and_no_input (w->window);
+
+	setup_task_menu (FOOBAR_WIDGET (w));
+
+	/* make sure we reset our position, so that if the wm misplaces us
+	 * we jump back */
+	gtk_widget_queue_resize (w);
 }
 
 static void
@@ -535,13 +604,255 @@ programs_menu_to_display(GtkWidget *menu)
 	}
 }
 
+static void
+set_the_task_submenu (FoobarWidget *foo, GtkWidget *item)
+{
+	foo->task_menu = gtk_menu_new ();
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), foo->task_menu);
+	/*g_message ("setting...");*/
+}
+
+static void
+focus_task (GtkWidget *w, GwmhTask *task)
+{
+	gwmh_desk_set_current_area (task->desktop, task->harea, task->varea);
+	gwmh_task_show  (task);
+	gwmh_task_raise (task);
+	gwmh_task_focus (task);
+}
+
+static void
+add_task (GwmhTask *task, FoobarWidget *foo)
+{
+	GtkWidget *item, *label;
+	char *title = NULL;
+	int slen;
+	GtkWidget *pixmap  = NULL;
+
+	static GwmhDesk *desk = NULL;
+
+	g_assert (foo->tasks);
+
+	if (GWMH_TASK_SKIP_WINLIST (task))
+		return;
+	/*g_message ("task: %s", task->name);*/
+	slen = strlen (task->name);
+	if (slen > 43)
+		title = g_strdup_printf ("%.20s...%s", task->name, task->name+slen-20);
+	item = gtk_pixmap_menu_item_new ();
+	pixmap = get_task_icon (task, GTK_WIDGET (foo));
+	if (pixmap != NULL) {
+		gtk_widget_show (pixmap);
+		gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM (item),
+						 pixmap);
+	}
+
+	label = gtk_label_new (title ? title : task->name);
+	g_free (title);
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_container_add (GTK_CONTAINER (item), label);
+	g_hash_table_insert (foo->tasks, task, item);
+	gtk_signal_connect (GTK_OBJECT (item), "activate", 
+			    GTK_SIGNAL_FUNC (focus_task),
+			    task);
+	gtk_widget_show_all (item);
+	
+	if (!desk)
+		desk = gwmh_desk_get_config ();
+
+	if (task->desktop == desk->current_desktop &&
+	    task->harea   == desk->current_harea   &&
+	    task->varea   == desk->current_varea)
+		gtk_menu_prepend (GTK_MENU (foo->task_menu), item);
+	else
+		gtk_menu_append (GTK_MENU (foo->task_menu), item);
+}
+
+static void
+create_task_menu (GtkWidget *w, gpointer data)
+{
+	FoobarWidget *foo = FOOBAR_WIDGET (data);
+	GList *tasks = gwmh_task_list_get ();
+
+	/*g_message ("creating...");*/
+	foo->tasks = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+	add_menu_separator (foo->task_menu);
+
+	g_list_foreach (tasks, (GFunc)add_task, foo);
+
+	/* Owen: don't read the next line */
+	GTK_MENU_SHELL (GTK_MENU_ITEM (w)->submenu)->active = 1;
+	our_gtk_menu_position (GTK_MENU (GTK_MENU_ITEM (w)->submenu));
+}
+
+static void
+destroy_task_menu (GtkWidget *w, gpointer data)
+{
+	FoobarWidget *foo = FOOBAR_WIDGET (data);
+	/*g_message ("removing...");*/
+	gtk_menu_item_remove_submenu (GTK_MENU_ITEM (w));
+	g_hash_table_destroy (foo->tasks);
+	foo->tasks = NULL;
+	set_the_task_submenu (foo, w);
+}
+
+static GtkWidget *
+get_default_pixmap (void)
+{
+	GtkWidget *widget;
+	static GdkPixmap *pixmap = NULL;
+	static GdkBitmap *mask   = NULL;
+	static gboolean looked   = FALSE;
+
+	if ( ! looked) {
+		GdkPixbuf *pb = NULL, *scaled = NULL;
+		pb = gdk_pixbuf_new_from_file (GNOME_ICONDIR"/gnome-tasklist.png");
+		
+		if (pb != NULL) {
+			scaled = gdk_pixbuf_scale_simple (pb, 20, 20, 
+							  GDK_INTERP_BILINEAR);
+			gdk_pixbuf_unref (pb);
+		}
+
+		if (scaled != NULL) {
+			gdk_pixbuf_render_pixmap_and_mask (scaled,
+							   &pixmap, &mask, 128);
+			gdk_pixbuf_unref (scaled);
+			
+			if (pixmap != NULL)
+				gdk_pixmap_ref (pixmap);
+
+			if (mask != NULL)
+				gdk_bitmap_ref (mask);
+		}
+
+		looked = TRUE;
+	}
+	if (pixmap != NULL)
+		widget = gtk_pixmap_new (pixmap, mask);
+	else
+		widget = gtk_label_new ("*");
+	gtk_widget_show (widget);
+
+	return widget;
+}
+
+static void
+set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
+{
+	if (!GTK_WIDGET_REALIZED (foo))
+		return;
+
+	if (foo->task_pixmap != NULL)
+		gtk_widget_destroy (foo->task_pixmap);
+	foo->task_pixmap = NULL;
+
+	if (task != NULL)
+		foo->task_pixmap = get_task_icon (task, GTK_WIDGET (foo));
+
+	if (foo->task_pixmap == NULL) {
+		foo->task_pixmap = get_default_pixmap ();
+	}
+
+	if (foo->task_pixmap != NULL) {
+		gtk_container_add (GTK_CONTAINER (foo->task_bin),
+				   foo->task_pixmap);
+	}
+}
+
+static gboolean
+task_notify (gpointer data, GwmhTask *task,
+	     GwmhTaskNotifyType ntype,
+	     GwmhTaskInfoMask imask)
+{
+	FoobarWidget *foo = FOOBAR_WIDGET (data);
+	GtkWidget *item;
+
+	switch (ntype) {
+	case GWMH_NOTIFY_INFO_CHANGED:
+		if (imask & GWMH_TASK_INFO_FOCUSED &&
+		    GWMH_TASK_FOCUSED (task)) {
+			set_das_pixmap (foo, task);
+		}
+		if (imask & GWMH_TASK_INFO_WM_HINTS &&
+		    GWMH_TASK_FOCUSED (task))
+			set_das_pixmap (foo, task);
+		break;
+	case GWMH_NOTIFY_NEW:
+		if (foo->tasks != NULL)
+			add_task (task, foo);
+		break;
+	case GWMH_NOTIFY_DESTROY:
+#warning Whoa; leak?
+		if (foo->tasks != NULL) {
+			item = g_hash_table_lookup (foo->tasks, task);
+			if (item) {
+				g_hash_table_remove (foo->tasks, task);
+				gtk_widget_hide (item);
+			} else {
+				g_warning ("Could not find item for task '%s'",
+					   task->name);
+			}
+		}
+		break;
+	default:
+		break;
+	}
+	return TRUE;
+}
+
+static void
+append_task_menu (FoobarWidget *foo, GtkMenuBar *menu_bar)
+{
+	foo->task_item = gtk_menu_item_new ();
+
+	foo->task_bin = gtk_alignment_new (0.3, 0.5, 0.0, 0.0);
+	gtk_widget_set_usize (foo->task_bin, 25, 20);
+	gtk_widget_show (foo->task_bin);
+	gtk_container_add (GTK_CONTAINER (foo->task_item), foo->task_bin);
+
+	gtk_menu_bar_append (menu_bar, foo->task_item);
+}
+
+static void
+setup_task_menu (FoobarWidget *foo)
+{
+	GList *tasks;
+	g_assert (foo->task_item != NULL);
+
+	gtk_signal_connect (GTK_OBJECT (foo->task_item), "select",
+			    GTK_SIGNAL_FUNC (create_task_menu), foo);
+	gtk_signal_connect (GTK_OBJECT (foo->task_item), "deselect",
+			    GTK_SIGNAL_FUNC (destroy_task_menu), foo);
+
+	set_the_task_submenu (foo, foo->task_item);
+
+	/* setup the pixmap to the focused task */
+	tasks = gwmh_task_list_get ();
+	while (tasks != NULL) {
+		if (GWMH_TASK_FOCUSED (tasks->data)) {
+			set_das_pixmap  (foo, tasks->data);
+			break;
+		}
+		tasks = tasks->next;
+	}
+
+	/* if no focused task found, then just set it to default */
+	if (tasks == NULL)
+		set_das_pixmap  (foo, NULL);
+
+	foo->notify = gwmh_task_notifier_add (task_notify, foo);
+}
+
 static void
 foobar_widget_init (FoobarWidget *foo)
 {
 	/*gchar *path;*/
 	GtkWindow *window = GTK_WINDOW (foo);
 	/*GtkWidget *bufmap;*/
-	GtkWidget *menu_bar;
+	GtkWidget *menu_bar, *bar;
 	GtkWidget *menu, *menuitem;
 	/*GtkWidget *align;*/
 	gint flags;
@@ -628,11 +939,15 @@ foobar_widget_init (FoobarWidget *foo)
 	gtk_box_pack_end (GTK_BOX (foo->hbox), align, FALSE, FALSE, 0);
 #endif
 
-	menu_bar = gtk_menu_bar_new ();
+	bar = menu_bar = gtk_menu_bar_new ();
 	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (menu_bar),
 				      GTK_SHADOW_NONE);
 	append_clock_menu (foo, menu_bar);
+#if 0
+	/* TODO: use the gnome menu if no gnome compliant WM or tasklist disabled */
 	append_gnome_menu (foo, menu_bar);
+#endif
+	append_task_menu (foo, GTK_MENU_BAR (bar));
 
 
 	gtk_box_pack_end (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
@@ -674,7 +989,14 @@ foobar_widget_destroy (GtkObject *o)
 	g_free (foo->clock_format);
 	foo->clock_format = NULL;
 
-	g_list_foreach (panel_list, queue_panel_resize, NULL);
+	g_slist_foreach (panel_list, queue_panel_resize, NULL);
+
+	if (foo->tasks != NULL)
+		g_hash_table_destroy (foo->tasks);
+	foo->tasks = NULL;
+	if (foo->notify > 0)
+		gwmh_task_notifier_remove (foo->notify);
+	foo->notify = 0;
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		GTK_OBJECT_CLASS (parent_class)->destroy (o);
@@ -687,7 +1009,7 @@ foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 		GTK_WIDGET_CLASS (parent_class)->size_allocate (w, alloc);
 
 	if (GTK_WIDGET_REALIZED (w))
-		g_list_foreach (panel_list, queue_panel_resize, NULL);
+		g_slist_foreach (panel_list, queue_panel_resize, NULL);
 }
 
 GtkWidget *
@@ -802,7 +1124,7 @@ foobar_widget_redo_window(FoobarWidget *foo)
 	newwin = gdk_window_new(NULL, &attributes, attributes_mask);
 	gdk_window_set_user_data(newwin, window);
 
-	xstuff_set_no_group(newwin);
+	xstuff_set_no_group_and_no_input (newwin);
 
 	/* reparent our main panel window */
 	gdk_window_reparent(foo->ebox->window, newwin, 0, 0);
