@@ -38,8 +38,16 @@ struct _FoobarWidget
 	GtkWidget *favorites;
 	GtkWidget *settings;
 
+	GHashTable *tasks;
+	GtkWidget *task_item;
+	GtkWidget *task_menu;
+	GtkWidget *task_pixmap;
+	GtkWidget *task_bin;
+
 	gboolean compliant_wm;
 	char *clock_format;
+
+	guint notify;
 };
 
 struct _FoobarWidgetClass
@@ -62,6 +70,8 @@ gint		foobar_widget_get_height	(void);
 
 void		foobar_widget_force_menu_remake	(void);
 
+void		foobar_widget_global_set_clock_format (const char *format);
+
 END_GNOME_DECLS
 
 #endif
