@@ -26,6 +26,7 @@
 #include <errno.h>
 
 #include "panel-include.h"
+#include "foobar-widget.h"
 
 #include "gnome-run.h"
 
@@ -44,11 +45,7 @@ string_callback (GtkWidget *w, int button_num, gpointer data)
 	GSList *tofree = NULL;
 
 	if (button_num == 2/*help*/) {
-		GnomeHelpMenuEntry help_entry = {
-			"panel",
-			"specialobjects.html#RUNBUTTON"
-		};
-		gnome_help_display(NULL, &help_entry);
+		panel_show_help ("specialobjects.html#RUNBUTTON");
 		/* just return as we don't want to close */
 		return;
 	} else if (button_num == 1/*cancel*/) {
@@ -61,14 +58,46 @@ string_callback (GtkWidget *w, int button_num, gpointer data)
 
 	s = gtk_entry_get_text(entry);
 
-	if (!s || !*s)
+	if (string_empty (s))
 		goto return_and_close;
 
+	/* evil eggies, do not translate! */
+	if (strcmp (s, "time shall be unixey") == 0) {
+		foobar_widget_global_set_clock_format ("%s");
+		goto return_and_close;
+	}
+	if (strcmp (s, "you shall bring us a shrubbery") == 0) {
+		gnome_ok_dialog ("NI! NI! NI! NI! NI! NI!");
+		goto return_and_close;
+	}
+	if (strcmp (s, "supreme executive power") == 0) {
+		gnome_ok_dialog ("Listen -- strange women lying in\n"
+				 "ponds distributing swords is no\n"
+				 "basis for a system of government.\n"
+				 "Supreme executive power derives from\n"
+				 "a mandate from the masses, not from\n"
+				 "some farcical aquatic ceremony!");
+		goto return_and_close;
+	}
+
+	/* Somewhat of a hack I suppose */
+	if (strncmp (s, "http://", strlen ("http://")) == 0 ||
+	    strncmp (s, "https://", strlen ("https://")) == 0 ||
+	    strncmp (s, "gopher://", strlen ("gopher://")) == 0 ||
+	    strncmp (s, "ftp://", strlen ("ftp://")) == 0 ||
+	    strncmp (s, "file:", strlen ("file:")) == 0 ||
+	    strncmp (s, "ghelp:", strlen ("ghelp:")) == 0 ||
+	    strncmp (s, "man:", strlen ("man:")) == 0 ||
+	    strncmp (s, "info:", strlen ("info:")) == 0) {
+		gnome_url_show (s);
+		goto return_and_close;
+	}
+
 	/* we use a popt function as it does exactly what we want to do and
 	   gnome already uses popt */
-	if(poptParseArgvString(s, &temp_argc, &temp_argv) != 0) {
-		panel_error_dialog(_("Failed to execute command:\n"
-				     "%s"), s);
+	if (poptParseArgvString (s, &temp_argc, &temp_argv) != 0) {
+		panel_error_dialog (_("Failed to execute command:\n"
+				      "%s"), s);
 		goto return_and_close;
 	}
 
@@ -94,7 +123,7 @@ string_callback (GtkWidget *w, int button_num, gpointer data)
 		} else {
 			char *check;
 			int i;
-			check = gnome_is_program_in_path("gnome-terminal");
+			check = panel_is_program_in_path("gnome-terminal");
 			argv = g_new(char *, 2 + temp_argc + 1);
 			tofree = g_slist_prepend(tofree, argv);
 			argc = 2 + temp_argc;
@@ -128,7 +157,7 @@ string_callback (GtkWidget *w, int button_num, gpointer data)
 			     "%s"),
 			   s, g_unix_error_string(errno));
 return_and_close:
-	gtk_widget_destroy(w);
+	gnome_dialog_close (GNOME_DIALOG (w));
 }
 
 static void
@@ -142,14 +171,14 @@ browse_ok(GtkWidget *widget, GtkFileSelection *fsel)
 	entry = gtk_object_get_user_data(GTK_OBJECT(fsel));
 
 	fname = gtk_file_selection_get_filename(fsel);
-	if(fname) {
-		char *s = gtk_entry_get_text(GTK_ENTRY(entry));
-		if(!s || !*s)
-			gtk_entry_set_text(GTK_ENTRY(entry), fname);
-		else {
-			s = g_strconcat(s, " ", fname, NULL);
-			gtk_entry_set_text(GTK_ENTRY(entry), s);
-			g_free(s);
+	if(fname != NULL) {
+		char *s = gtk_entry_get_text (GTK_ENTRY (entry));
+		if (string_empty (s)) {
+			gtk_entry_set_text (GTK_ENTRY (entry), fname);
+		} else {
+			s = g_strconcat (s, " ", fname, NULL);
+			gtk_entry_set_text (GTK_ENTRY (entry), s);
+			g_free (s);
 		}
 	}
 	gtk_widget_destroy(GTK_WIDGET(fsel));
@@ -271,9 +300,32 @@ show_run_dialog_with_text (const char *text)
 	gtk_entry_set_text(GTK_ENTRY(entry), text);
 }
 
+static void  
+drag_data_get_cb (GtkWidget          *widget,
+		  GdkDragContext     *context,
+		  GtkSelectionData   *selection_data,
+		  guint               info,
+		  guint               time,
+		  gpointer            data)
+{
+	char *foo;
+
+	foo = g_strdup_printf ("RUN:%d", find_applet (widget));
+
+	gtk_selection_data_set (selection_data,
+				selection_data->target, 8, (guchar *)foo,
+				strlen (foo));
+
+	g_free (foo);
+}
+
+
 static GtkWidget *
 create_run_widget(void)
 {
+        static GtkTargetEntry dnd_targets[] = {
+		{ "application/x-panel-applet-internal", 0, 0 }
+	};
 	GtkWidget *button;
 	char *pixmap_name;
 
@@ -284,6 +336,23 @@ create_run_widget(void)
 				   FALSE,
 				   ORIENT_UP,
 				   _("Run..."));
+
+	/*A hack since this function only pretends to work on window
+	  widgets (which we actually kind of are) this will select
+	  some (already selected) events on the panel instead of
+	  the button window (where they are also selected) but
+	  we don't mind*/
+	GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
+	gtk_drag_source_set (button,
+			     GDK_BUTTON1_MASK,
+			     dnd_targets, 1,
+			     GDK_ACTION_COPY | GDK_ACTION_MOVE);
+	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
+
+	gtk_signal_connect (GTK_OBJECT (button), "drag_data_get",
+			    GTK_SIGNAL_FUNC (drag_data_get_cb),
+			    NULL);
+
 	g_free(pixmap_name);
 	gtk_tooltips_set_tip (panel_tooltips, button, _("Run..."), NULL);
 
@@ -302,7 +371,7 @@ load_run_applet(PanelWidget *panel, int pos, gboolean exactpos)
 	if(!run)
 		return;
 
-	if (!register_toy(run, NULL, panel,
+	if (!register_toy(run, NULL, NULL, panel,
 			  pos, exactpos, APPLET_RUN))
 		return;
 
