@@ -13,6 +13,7 @@ typedef struct _DirRec DirRec;
 struct _MenuFinfo {
 	char *menudir;
 	gboolean applets;
+	gboolean launcher_add;
 	char *dir_name;
 	char *pixmap_name;	
 	gboolean fake_menu;
@@ -34,6 +35,7 @@ struct _FileRec {
 	char *fullname;
 	char *icon;
 	char *goad_id;
+	char *tryexec_path;
 	DirRec *parent;
 	time_t mtime;
 	time_t last_stat;
@@ -45,13 +47,14 @@ struct _DirRec {
 	time_t dentrylast_stat;
 	time_t merge_mtime;
 	gboolean force_reread;
+	GSList *tryexecs;
 	GSList *recs; /*records for directories*/
 	GSList *mfl;  /*records of menus using this record*/
 };
 
 char * get_applet_goad_id_from_dentry(GnomeDesktopEntry *ii);
 
-GSList * get_files_from_menudir(char *menudir);
+GSList * get_files_from_menudir(const char *menudir);
 
 FileRec * fr_read_dir(DirRec *dr, const char *mdir, struct stat *dstat, struct stat *merge_dstat, int sublevels);
 FileRec * fr_replace(FileRec *fr);
