@@ -20,6 +20,10 @@
 #include "gnome-run.h"
 #include "gnome-panel.h"
 
+/* nautilus uses this UTTER HACK to reset backgrounds, ugly ugly ugly,
+ * broken, ugly ugly, but whatever */
+#define RESET_IMAGE_NAME "/nautilus/backgrounds/reset.png"
+
 #define PANEL_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
 			   GDK_BUTTON_RELEASE_MASK |		\
 			   GDK_POINTER_MOTION_MASK |		\
@@ -28,14 +32,17 @@
 /*list of all panel widgets created*/
 GSList *panel_list = NULL;
 
-static int panel_dragged = FALSE;
+static gboolean panel_dragged = FALSE;
 static int panel_dragged_timeout = -1;
-static int panel_been_moved = FALSE;
+static gboolean panel_been_moved = FALSE;
 
 /*the number of base panels (corner/snapped) out there, never let it
   go below 1*/
 int base_panels = 0;
 
+extern GSList *applets;
+extern GSList *applets_last;
+
 extern int config_sync_timeout;
 extern int applets_to_sync;
 extern int panels_to_sync;
@@ -59,6 +66,7 @@ enum {
 	TARGET_COLOR,
 	TARGET_APPLET,
 	TARGET_APPLET_INTERNAL,
+	TARGET_ICON_INTERNAL,
 	TARGET_BGIMAGE
 };
 
@@ -70,6 +78,7 @@ static GtkTargetEntry panel_drop_types[] = {
 	{ "application/x-panel-directory", 0, TARGET_DIRECTORY },
 	{ "application/x-panel-applet", 0, TARGET_APPLET },
 	{ "application/x-panel-applet-internal", 0, TARGET_APPLET_INTERNAL },
+	{ "application/x-panel-icon-internal", 0, TARGET_ICON_INTERNAL },
 	{ "application/x-color", 0, TARGET_COLOR },
 	{ "property/bgimage",    0, TARGET_BGIMAGE }
 };
@@ -89,9 +98,9 @@ change_window_cursor(GdkWindow *window, GdkCursorType cursor_type)
 }
 
 static void
-panel_realize(GtkWidget *widget, gpointer data)
+panel_realize (GtkWidget *widget, gpointer data)
 {
-	change_window_cursor(widget->window, GDK_LEFT_PTR);
+	change_window_cursor (widget->window, GDK_LEFT_PTR);
 	
 	if (IS_BASEP_WIDGET (widget))
 		basep_widget_enable_buttons(BASEP_WIDGET(widget));
@@ -100,41 +109,49 @@ panel_realize(GtkWidget *widget, gpointer data)
 
 	/*FIXME: this seems to fix the panel size problems on startup
 	  (from a report) but I don't think it's right*/
-	gtk_widget_queue_resize(GTK_WIDGET(widget));
+	gtk_widget_queue_resize (GTK_WIDGET (widget));
 }
 
-static void
-freeze_changes(AppletInfo *info)
+void
+freeze_changes (AppletInfo *info)
 {
 	if(info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
-		g_assert(ext);
+		g_assert (ext != NULL);
 		/*ingore this until we get an ior*/
-		if(ext->applet) {
+		if (ext->applet != CORBA_OBJECT_NIL) {
 			CORBA_Environment ev;
 			CORBA_exception_init(&ev);
 			GNOME_Applet_freeze_changes(ext->applet, &ev);
-			if(ev._major)
-				panel_clean_applet(ext->info);
+			if(ev._major) {
+				extern_ref (ext);
+				panel_clean_applet(info);
+				ext->info = NULL;
+				extern_unref (ext);
+			}
 			CORBA_exception_free(&ev);
 		}
 	}
 }
 
-static void
+void
 thaw_changes(AppletInfo *info)
 {
 	if(info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
-		g_assert(ext);
+		g_assert (ext != NULL);
 		/*ingore this until we get an ior*/
-		if(ext->applet) {
+		if (ext->applet != CORBA_OBJECT_NIL) {
 			CORBA_Environment ev;
-			CORBA_exception_init(&ev);
-			GNOME_Applet_thaw_changes(ext->applet, &ev);
-			if(ev._major)
-				panel_clean_applet(ext->info);
-			CORBA_exception_free(&ev);
+			CORBA_exception_init (&ev);
+			GNOME_Applet_thaw_changes (ext->applet, &ev);
+			if(ev._major) {
+				extern_ref (ext);
+				panel_clean_applet (info);
+				ext->info = NULL;
+				extern_unref (ext);
+			}
+			CORBA_exception_free (&ev);
 		}
 	}
 }
@@ -200,8 +217,12 @@ orientation_change(AppletInfo *info, PanelWidget *panel)
 			GNOME_Applet_change_orient(ext->applet,
 						   orient,
 						   &ev);
-			if(ev._major)
-				panel_clean_applet(ext->info);
+			if(ev._major) {
+				extern_ref (ext);
+				panel_clean_applet(info);
+				ext->info = NULL;
+				extern_unref (ext);
+			}
 			CORBA_exception_free(&ev);
 
 			/* we have now sent this orientation thus we
@@ -292,8 +313,12 @@ size_change(AppletInfo *info, PanelWidget *panel)
 			GNOME_Applet_change_size(ext->applet,
 						 panel->sz,
 						 &ev);
-			if(ev._major)
-				panel_clean_applet(ext->info);
+			if(ev._major) {
+				extern_ref (ext);
+				panel_clean_applet(info);
+				ext->info = NULL;
+				extern_unref (ext);
+			}
 			CORBA_exception_free(&ev);
 		}
 	} else if(info->type == APPLET_STATUS) {
@@ -322,11 +347,11 @@ panel_size_change(GtkWidget *widget,
 			      widget);
 	panels_to_sync = TRUE;
 	/*update the configuration box if it is displayed*/
-	update_config_size(PANEL_WIDGET(widget)->panel_parent);
+	update_config_size (PANEL_WIDGET (widget)->panel_parent);
 }
 
 void
-back_change(AppletInfo *info, PanelWidget *panel)
+back_change (AppletInfo *info, PanelWidget *panel)
 {
 	if(info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
@@ -345,8 +370,12 @@ back_change(AppletInfo *info, PanelWidget *panel)
 				backing._u.c.blue = panel->back_color.blue;
 			}
 			GNOME_Applet_back_change(ext->applet, &backing, &ev);
-			if(ev._major)
-				panel_clean_applet(ext->info);
+			if (ev._major) {
+				extern_ref (ext);
+				panel_clean_applet (info);
+				ext->info = NULL;
+				extern_unref (ext);
+			}
 			CORBA_exception_free(&ev);
 		}
 	} 
@@ -546,7 +575,7 @@ move_panel_to_cursor(GtkWidget *w)
 		basep_widget_set_pos(BASEP_WIDGET(w),x,y);
 }
 
-static int
+static gboolean
 panel_move_timeout(gpointer data)
 {
 	if(panel_dragged && panel_been_moved)
@@ -559,25 +588,51 @@ panel_move_timeout(gpointer data)
 }
 
 static void
-panel_destroy(GtkWidget *widget, gpointer data)
+clean_kill_applets (PanelWidget *panel)
 {
-	PanelData *pd = gtk_object_get_user_data(GTK_OBJECT(widget));
+	GList *li;
+	for (li = panel->applet_list; li != NULL; li = li->next) {
+		AppletData *ad = li->data;
+		AppletInfo *info =
+			gtk_object_get_data (GTK_OBJECT (ad->applet),
+					     "applet_info");
+		if (info->type == APPLET_EXTERN) {
+			Extern *ext = info->data;
+			ext->clean_remove = TRUE;
+		} else if (info->type == APPLET_SWALLOW) {
+			Swallow *swallow = info->data;
+			swallow->clean_remove = TRUE;
+		}
+	}
+}
+
+static void
+panel_destroy (GtkWidget *widget, gpointer data)
+{
+	PanelData *pd = gtk_object_get_user_data (GTK_OBJECT (widget));
 	PanelWidget *panel = NULL;
 
 	if (IS_BASEP_WIDGET (widget))
 		panel = PANEL_WIDGET(BASEP_WIDGET(widget)->panel);
 	else if (IS_FOOBAR_WIDGET (widget))
 		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
+
+	clean_kill_applets (panel);
 		
 	kill_config_dialog(widget);
 
-	if(IS_DRAWER_WIDGET(widget)) {
-		if(panel->master_widget) {
-			AppletInfo *info = gtk_object_get_data(GTK_OBJECT(panel->master_widget),
-							       "applet_info");
+	if (IS_DRAWER_WIDGET(widget)) {
+		GtkWidget *master_widget = panel->master_widget;
+
+		if (master_widget != NULL) {
+			AppletInfo *info =
+				gtk_object_get_data(GTK_OBJECT(master_widget),
+						    "applet_info");
 			Drawer *drawer = info->data;
 			drawer->drawer = NULL;
-			panel_clean_applet(info);
+			panel_clean_applet (info);
+			gtk_object_remove_data (GTK_OBJECT (master_widget),
+						"applet_info");
 		}
 	} else if ((IS_BASEP_WIDGET(widget)
 		    && !IS_DRAWER_WIDGET(widget))
@@ -586,21 +641,12 @@ panel_destroy(GtkWidget *widget, gpointer data)
 		base_panels--;
 	}
 
-/* don't unref this, since we unref from a signal connected to the
- * destroy signal of the panel
- *
- * Broken, maybe.  1.2 needs to go out, definitely.
- *
- * Oh, and null it anyway since we should be done with it.
- */
-#if 0	
-	if(pd->menu)
-		gtk_widget_unref(pd->menu);
-#endif
+	if (pd->menu != NULL)
+		gtk_widget_unref (pd->menu);
 	pd->menu = NULL;
 
-	panel_list = g_slist_remove(panel_list,pd);
-	g_free(pd);
+	panel_list = g_slist_remove (panel_list, pd);
+	g_free (pd);
 }
 
 static void
@@ -637,14 +683,14 @@ panel_applet_about_to_die (GtkWidget *panel, GtkWidget *widget, gpointer data)
 }
 
 static GtkWidget *
-panel_menu_get(PanelWidget *panel, PanelData *pd)
+panel_menu_get (PanelWidget *panel, PanelData *pd)
 {
-	if(pd->menu)
+	if (pd->menu != NULL)
 		return pd->menu;
 	
-	pd->menu = create_panel_root_menu(panel, TRUE);
-	gtk_signal_connect(GTK_OBJECT(pd->menu), "deactivate",
-			   GTK_SIGNAL_FUNC(menu_deactivate), pd);
+	pd->menu = create_panel_root_menu (panel, TRUE);
+	gtk_signal_connect (GTK_OBJECT (pd->menu), "deactivate",
+			    GTK_SIGNAL_FUNC (menu_deactivate), pd);
 	return pd->menu;
 }
 
@@ -671,8 +717,54 @@ make_popup_panel_menu (PanelWidget *panel)
 	pd->menu_age = 0;
 	return menu;
 }
+
+static gboolean
+panel_initiate_move (GtkWidget *widget, guint32 event_time)
+{
+	PanelWidget *panel = NULL;
+	BasePWidget *basep = NULL;
+
+	if (IS_BASEP_WIDGET (widget)) {
+		basep = BASEP_WIDGET (widget);
+		panel = PANEL_WIDGET (basep->panel);
+	} else if (IS_FOOBAR_WIDGET (widget)) {
+		panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
+	}
+
+	/*this should probably be in snapped widget*/
+	if(!panel_dragged &&
+	   !IS_DRAWER_WIDGET (widget) &&
+	   !IS_FOOBAR_WIDGET (widget)) {
+		GdkCursor *cursor = gdk_cursor_new (GDK_FLEUR);
+		gtk_grab_add(widget);
+		gdk_pointer_grab (widget->window,
+				  FALSE,
+				  PANEL_EVENT_MASK,
+				  NULL,
+				  cursor,
+				  event_time);
+		gdk_cursor_destroy (cursor);
+
+		if (basep) {
+			basep->autohide_inhibit = TRUE;
+			basep_widget_init_offsets (basep);
+		}
+
+		panel_dragged = TRUE;
+		return TRUE;
+	} if(IS_DRAWER_WIDGET(widget) &&
+	     !panel_applet_in_drag) {
+		panel_widget_applet_drag_start (
+						PANEL_WIDGET(panel->master_widget->parent),
+						panel->master_widget,
+						PW_DRAG_OFF_CURSOR);
+		return TRUE;
+	}
+
+	return FALSE;
+}
 	
-static int
+static gboolean
 panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 {
 	PanelWidget *panel = NULL;
@@ -690,7 +782,7 @@ panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 	case GDK_BUTTON_PRESS:
 		bevent = (GdkEventButton *) event;
 		switch(bevent->button) {
-		case 3: /* fall through */
+		case 3:
 			if(!panel_applet_in_drag) {
 				GtkWidget *menu;
 
@@ -709,32 +801,7 @@ panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 			}
 			break;
 		case 2:
-			/*this should probably be in snapped widget*/
-			if(!panel_dragged &&
-			   !IS_DRAWER_WIDGET (widget) &&
-			   !IS_FOOBAR_WIDGET (widget)) {
-				GdkCursor *cursor = gdk_cursor_new (GDK_FLEUR);
-				gtk_grab_add(widget);
-				gdk_pointer_grab (widget->window,
-						  FALSE,
-						  PANEL_EVENT_MASK,
-						  NULL,
-						  cursor,
-						  bevent->time);
-				gdk_cursor_destroy (cursor);
-
-				if (basep)
-					basep->autohide_inhibit = TRUE;
-
-				panel_dragged = TRUE;
-				return TRUE;
-			} if(IS_DRAWER_WIDGET(widget) &&
-			     !panel_applet_in_drag) {
-				panel_widget_applet_drag_start (
-					PANEL_WIDGET(panel->master_widget->parent),
-					panel->master_widget);
-				return TRUE;
-			}
+			return panel_initiate_move (widget, bevent->time);
 			break;
 		default: break;
 		}
@@ -778,7 +845,21 @@ panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 	return FALSE;
 }
 
-static int
+static gboolean
+panel_widget_event (GtkWidget *widget, GdkEvent *event, GtkWidget *panelw)
+{
+	if (event->type == GDK_BUTTON_PRESS) {
+		GdkEventButton *bevent = (GdkEventButton *) event;
+
+		if (bevent->button == 1 ||
+		    bevent->button == 2)
+			return panel_initiate_move (panelw, bevent->time);
+	}
+
+	return FALSE;
+}
+
+static gboolean
 panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
 	GdkEventButton *bevent;
@@ -806,44 +887,45 @@ static gchar *
 extract_filename (const gchar* uri)
 {
 	/* file uri with a hostname */
-	if (strncmp(uri, "file://", strlen("file://"))==0) {
-		char *hostname = g_strdup(&uri[strlen("file://")]);
-		char *p = strchr(hostname,'/');
+	if (strncmp (uri, "file://", strlen ("file://")) == 0) {
+		char *hostname = g_strdup (&uri[strlen("file://")]);
+		char *p = strchr (hostname, '/');
 		char *path;
 		char localhostname[1024];
 		/* if we can't find the '/' this uri is bad */
-		if(!p) {
-			g_free(hostname);
+		if(p == NULL) {
+			g_free (hostname);
 			return NULL;
 		}
 		/* if no hostname */
-		if(p==hostname)
+		if(p == hostname)
 			return hostname;
 
-		path = g_strdup(p);
+		path = g_strdup (p);
 		*p = '\0';
 
 		/* if really local */
-		if(g_strcasecmp(hostname,"localhost")==0) {
-			g_free(hostname);
+		if (strcasecmp_no_locale (hostname, "localhost") == 0 ||
+		    strcasecmp_no_locale (hostname, "localhost.localdomain") == 0) {
+			g_free (hostname);
 			return path;
 		}
 
 		/* ok get the hostname */
-		if(gethostname(localhostname,
-			       sizeof(localhostname)) < 0) {
-			strcpy(localhostname,"");
+		if (gethostname (localhostname,
+				 sizeof (localhostname)) < 0) {
+			strcpy (localhostname, "");
 		}
 
 		/* if really local */
-		if(localhostname[0] &&
-		   g_strcasecmp(hostname,localhostname)==0) {
-			g_free(hostname);
+		if (localhostname[0] &&
+		    strcasecmp_no_locale (hostname, localhostname) == 0) {
+			g_free (hostname);
 			return path;
 		}
 		
-		g_free(hostname);
-		g_free(path);
+		g_free (hostname);
+		g_free (path);
 		return NULL;
 
 	/* if the file doesn't have the //, we take it containing 
@@ -860,14 +942,14 @@ extract_filename (const gchar* uri)
 static void
 drop_url(PanelWidget *panel, int pos, char *url)
 {
-	char *p = g_strdup_printf(_("Open URL: %s"),url);
-	load_launcher_applet_from_info_url(url, p, url, "netscape.png",
-					   panel, pos, TRUE);
+	char *p = g_strdup_printf (_("Open URL: %s"), url);
+	load_launcher_applet_from_info_url (url, p, url, "gnome-globe.png",
+					    panel, pos, TRUE);
 	g_free(p);
 }
 
 static void
-drop_menu(PanelWidget *panel, int pos, char *dir)
+drop_menu (PanelWidget *panel, int pos, const char *dir)
 {
 	int flags = MAIN_MENU_SYSTEM | MAIN_MENU_USER;
 	DistributionType distribution = get_distribution_type ();
@@ -876,33 +958,39 @@ drop_menu(PanelWidget *panel, int pos, char *dir)
 	if(distribution != DISTRIBUTION_UNKNOWN)
 		flags |= MAIN_MENU_DISTRIBUTION_SUB;
 	/* Guess KDE menus */
-	if(g_file_exists(kde_menudir))
+	if(panel_file_exists(kde_menudir))
 		flags |= MAIN_MENU_KDE_SUB;
-	load_menu_applet(dir, flags, TRUE, FALSE, NULL, panel, pos, TRUE);
+	load_menu_applet (dir, flags, TRUE, FALSE, NULL, panel, pos, TRUE);
 }
 
 static void
-drop_urilist(PanelWidget *panel, int pos, char *urilist,
-	     gboolean background_drops)
+drop_urilist (PanelWidget *panel, int pos, char *urilist,
+	      gboolean background_drops)
 {
 	GList *li, *files;
 	struct stat s;
 
 	files = gnome_uri_list_extract_uris(urilist);
 
-	for(li = files; li; li = g_list_next(li)) {
+	for(li = files; li; li = li->next) {
+		char *uri;
 		const char *mimetype;
 		char *filename;
 
-		if(strncmp(li->data, "http:", strlen("http:")) == 0 ||
-		   strncmp(li->data, "https:", strlen("https:")) == 0 ||
-		   strncmp(li->data, "ftp:", strlen("ftp:")) == 0 ||
-		   strncmp(li->data, "gopher:", strlen("gopher:")) == 0) {
-			drop_url(panel,pos,li->data);
+		uri = li->data;
+
+		if (strncmp (uri, "http:", strlen ("http:")) == 0 ||
+		    strncmp (uri, "https:", strlen ("https:")) == 0 ||
+		    strncmp (uri, "ftp:", strlen ("ftp:")) == 0 ||
+		    strncmp (uri, "gopher:", strlen ("gopher:")) == 0 ||
+		    strncmp (uri, "ghelp:", strlen ("ghelp:")) == 0 ||
+		    strncmp (uri, "man:", strlen ("man:")) == 0 ||
+		    strncmp (uri, "info:", strlen ("info:")) == 0) {
+			drop_url (panel, pos, uri);
 			continue;
 		}
 
-		filename = extract_filename(li->data);
+		filename = extract_filename (li->data);
 		if(filename == NULL)
 			continue;
 
@@ -913,61 +1001,170 @@ drop_urilist(PanelWidget *panel, int pos, char *urilist,
 
 		mimetype = gnome_mime_type(filename);
 
-		if(background_drops &&
-		   mimetype && !strncmp(mimetype, "image", sizeof("image")-1))
+		if (background_drops &&
+		    mimetype != NULL &&
+		    strncmp(mimetype, "image", sizeof("image")-1) == 0) {
 			panel_widget_set_back_pixmap (panel, filename);
-		else if(mimetype &&
-			(strcmp(mimetype, "application/x-gnome-app-info")==0 ||
-			 strcmp(mimetype, "application/x-kde-app-info")==0))
-			load_launcher_applet(filename, panel, pos, TRUE);
-		else if(S_ISDIR(s.st_mode)) {
+		} else if (mimetype != NULL &&
+			  (strcmp(mimetype, "application/x-gnome-app-info") == 0 ||
+			   strcmp(mimetype, "application/x-kde-app-info") == 0)) {
+			Launcher *launcher;
+
+			launcher = load_launcher_applet (filename, panel, pos, TRUE);
+
+			if (launcher != NULL)
+				launcher_hoard (launcher);
+		} else if (S_ISDIR(s.st_mode)) {
 			drop_menu(panel, pos, filename);
-		} else if(S_IEXEC & s.st_mode) /*executable?*/
-			ask_about_launcher(filename,panel,pos,TRUE);
-		g_free(filename);
+		} else if (S_IEXEC & s.st_mode) /*executable?*/
+			ask_about_launcher (filename, panel, pos, TRUE);
+		g_free (filename);
 	}
 
 	gnome_uri_list_free_strings (files);
 }
 
 static void
-drop_bgimage(PanelWidget *panel, char *bgimage)
+drop_bgimage (PanelWidget *panel, const char *bgimage)
 {
 	char *filename;
 
-	filename = extract_filename(bgimage);
-	if(filename != NULL) {
-		panel_widget_set_back_pixmap (panel, filename);
+	filename = extract_filename (bgimage);
+	if (filename != NULL) {
+		/* an incredible hack, no, worse, INCREDIBLE FUCKING HACK,
+		 * whatever, we need to work with nautilus on this one */
+		if (strstr (filename, RESET_IMAGE_NAME) != NULL) {
+			panel_widget_change_params (panel,
+						    panel->orient,
+						    panel->sz,
+						    PANEL_BACK_NONE,
+						    panel->back_pixmap,
+						    panel->fit_pixmap_bg,
+						    panel->strech_pixmap_bg,
+						    panel->rotate_pixmap_bg,
+						    panel->no_padding_on_ends,
+						    &panel->back_color);
+		} else {
+			panel_widget_set_back_pixmap (panel, filename);
+		}
 
-		g_free(filename);
+		g_free (filename);
 	}
 }
 
 static void
-drop_internal_applet(PanelWidget *panel, int pos, char *applet_type)
+drop_internal_icon (PanelWidget *panel, int pos, const char *icon_name,
+		    int action)
 {
-	if(!applet_type)
+	Launcher *old_launcher, *launcher;
+
+	if (icon_name == NULL)
 		return;
-	if(strncmp(applet_type,"MENU:",strlen("MENU:"))==0) {
-		char *menu = &applet_type[strlen("MENU:")];
-		if(strcmp(menu,"MAIN")==0)
-			drop_menu(panel, pos, NULL);
+
+	if (action == GDK_ACTION_MOVE) {
+		old_launcher = find_launcher (icon_name);
+	} else {
+		old_launcher = NULL;
+	}
+
+	launcher = load_launcher_applet (icon_name, panel, pos, TRUE);
+
+	if (launcher != NULL) {
+		launcher_hoard (launcher);
+
+		if (old_launcher != NULL &&
+		    old_launcher->button != NULL)
+			gtk_widget_destroy (old_launcher->button);
+	}
+}
+
+static void
+move_applet (PanelWidget *panel, int pos, int applet_num)
+{
+	AppletInfo *info = g_slist_nth_data (applets, applet_num);
+
+	if (pos < 0)
+		pos = 0;
+
+	if (info != NULL &&
+	    info->widget != NULL &&
+	    info->widget->parent != NULL &&
+	    IS_PANEL_WIDGET (info->widget->parent)) {
+		GSList *forb;
+		forb = gtk_object_get_data (GTK_OBJECT (info->widget),
+					    PANEL_APPLET_FORBIDDEN_PANELS);
+		if ( ! g_slist_find (forb, panel))
+			panel_widget_reparent (PANEL_WIDGET (info->widget->parent),
+					       panel,
+					       info->widget,
+					       pos);
+	}
+}
+
+static void
+drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
+		      int action)
+{
+	int applet_num = -1;
+	gboolean remove_applet = FALSE;
+
+	if (applet_type == NULL)
+		return;
+
+	if (sscanf (applet_type, "MENU:%d", &applet_num) == 1 ||
+	    sscanf (applet_type, "DRAWER:%d", &applet_num) == 1 ||
+	    sscanf (applet_type, "SWALLOW:%d", &applet_num) == 1) {
+		if (action != GDK_ACTION_MOVE)
+			g_warning ("Only MOVE supported for menus/drawers/swallows");
+		move_applet (panel, pos, applet_num);
+
+	} else if (strncmp (applet_type, "MENU:", strlen("MENU:")) == 0) {
+		const char *menu = &applet_type[strlen ("MENU:")];
+		if (strcmp (menu, "MAIN") == 0)
+			drop_menu (panel, pos, NULL);
 		else
-			drop_menu(panel, pos, menu);
-	} else if(strcmp(applet_type,"DRAWER:NEW")==0) {
+			drop_menu (panel, pos, menu);
+
+	} else if (strcmp(applet_type,"DRAWER:NEW")==0) {
 		load_drawer_applet(-1, NULL, NULL, panel, pos, TRUE);
-	} else if(strcmp(applet_type,"LOGOUT:NEW")==0) {
-		load_logout_applet(panel, pos, TRUE);
-	} else if(strcmp(applet_type,"LOCK:NEW")==0) {
-		load_lock_applet(panel, pos, TRUE);
-	} else if(strcmp(applet_type,"SWALLOW:ASK")==0) {
+
+	} else if (strcmp (applet_type, "LOGOUT:NEW") == 0) {
+		load_logout_applet (panel, pos, TRUE);
+
+	} else if (sscanf (applet_type, "LOGOUT:%d", &applet_num) == 1) {
+		load_logout_applet (panel, pos, TRUE);
+		remove_applet = TRUE;
+
+	} else if (strcmp (applet_type, "LOCK:NEW") == 0) {
+		load_lock_applet (panel, pos, TRUE);
+
+	} else if (sscanf (applet_type, "LOCK:%d", &applet_num) == 1) {
+		load_lock_applet (panel, pos, TRUE);
+		remove_applet = TRUE;
+
+	} else if (strcmp (applet_type, "SWALLOW:ASK") == 0) {
 		ask_about_swallowing(panel, pos, TRUE);
+
 	} else if(strcmp(applet_type,"LAUNCHER:ASK")==0) {
 		ask_about_launcher(NULL, panel, pos, TRUE);
+
 	} else if(strcmp(applet_type,"STATUS:TRY")==0) {
 		load_status_applet(panel, pos, TRUE);
+
 	} else if(strcmp(applet_type,"RUN:NEW")==0) {
 		load_run_applet(panel, pos, TRUE);
+
+	} else if (sscanf (applet_type, "RUN:%d", &applet_num) == 1) {
+		load_run_applet(panel, pos, TRUE);
+		remove_applet = TRUE;
+	}
+
+	if (remove_applet &&
+	    action == GDK_ACTION_MOVE) {
+		AppletInfo *info = g_slist_nth_data (applets, applet_num);
+
+		if (info != NULL)
+			panel_clean_applet (info);
 	}
 }
 
@@ -987,16 +1184,15 @@ drop_color(PanelWidget *panel, int pos, guint16 *dropped)
 }
 
 static gboolean
-is_this_drop_ok(GtkWidget *widget, GdkDragContext *context, guint *info,
-		GdkAtom *ret_atom)
+is_this_drop_ok (GtkWidget *widget, GdkDragContext *context, guint *info,
+		 GdkAtom *ret_atom)
 {
-	GtkWidget *source_widget;
 	GtkWidget *panel; /*PanelWidget*/
 	GList *li;
 
-	g_return_val_if_fail(widget!=NULL, FALSE);
-	g_return_val_if_fail(IS_BASEP_WIDGET (widget) ||
-			     IS_FOOBAR_WIDGET (widget), FALSE);
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (IS_BASEP_WIDGET (widget) ||
+			      IS_FOOBAR_WIDGET (widget), FALSE);
 
 	if(!(context->actions & (GDK_ACTION_COPY|GDK_ACTION_MOVE)))
 		return FALSE;
@@ -1006,33 +1202,27 @@ is_this_drop_ok(GtkWidget *widget, GdkDragContext *context, guint *info,
 	else
 		panel = FOOBAR_WIDGET (widget)->panel;
 
-	source_widget = gtk_drag_get_source_widget (context);
-	/* if this is a drag from this panel to this panel of a launcher,
-	   then ignore it */
-	if(source_widget && IS_BUTTON_WIDGET(source_widget) &&
-	   source_widget->parent == panel) {
-		return FALSE;
-	}
-
-	if(!panel_target_list)
-		panel_target_list = gtk_target_list_new(panel_drop_types,
-							n_panel_drop_types);
-	for(li = context->targets; li; li = li->next) {
+	if (panel_target_list == NULL)
+		panel_target_list = gtk_target_list_new (panel_drop_types,
+							 n_panel_drop_types);
+	for (li = context->targets; li; li = li->next) {
 		guint temp_info;
-		if(gtk_target_list_find(panel_target_list, 
-					GPOINTER_TO_UINT(li->data),
-					&temp_info)) {
-			if((temp_info == TARGET_COLOR ||
-			    temp_info == TARGET_BGIMAGE) &&
-			   IS_FOOBAR_WIDGET(widget))
+		if (gtk_target_list_find (panel_target_list, 
+					  GPOINTER_TO_UINT(li->data),
+					  &temp_info)) {
+			if ((temp_info == TARGET_COLOR ||
+			     temp_info == TARGET_BGIMAGE) &&
+			    IS_FOOBAR_WIDGET (widget))
 				return FALSE;
-			if(info) *info = temp_info;
-			if(ret_atom) *ret_atom = GPOINTER_TO_UINT(li->data);
+			if (info != NULL)
+				*info = temp_info;
+			if (ret_atom != NULL)
+				*ret_atom = GPOINTER_TO_UINT(li->data);
 			break;
 		}
 	}
 	/* if we haven't found it */
-	if(!li)
+	if (li == NULL)
 		return FALSE;
 	return TRUE;
 }
@@ -1060,42 +1250,78 @@ do_highlight (GtkWidget *widget, gboolean highlight)
 
 
 static gboolean
-drag_motion_cb(GtkWidget	  *widget,
-	       GdkDragContext     *context,
-	       gint                x,
-	       gint                y,
-	       guint               time)
+drag_motion_cb (GtkWidget	   *widget,
+		GdkDragContext     *context,
+		gint                x,
+		gint                y,
+		guint               time)
 {
-	/* always prefer copy */
-	if(context->actions & GDK_ACTION_COPY)
-		gdk_drag_status (context, GDK_ACTION_COPY, time);
-	else
-		gdk_drag_status (context, context->suggested_action, time);
+	guint info;
 
-	if(!is_this_drop_ok(widget, context, NULL, NULL))
+	if ( ! is_this_drop_ok (widget, context, &info, NULL))
 		return FALSE;
 
-	do_highlight(widget, TRUE);
+	/* check forbiddenness */
+	if (info == TARGET_APPLET_INTERNAL) {
+		GtkWidget *source_widget;
+
+		source_widget = gtk_drag_get_source_widget (context);
+		if (source_widget != NULL &&
+		    IS_BUTTON_WIDGET (source_widget)) {
+			GSList *forb;
+			PanelWidget *panel = NULL;
+
+			if (IS_BASEP_WIDGET (widget)) {
+				BasePWidget *basep =
+					BASEP_WIDGET (widget);
+				panel = PANEL_WIDGET (basep->panel);
+			} else if (IS_FOOBAR_WIDGET (widget)) {
+				panel = PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel);
+			}
+			forb = gtk_object_get_data (GTK_OBJECT (source_widget),
+						    PANEL_APPLET_FORBIDDEN_PANELS);
+			if (panel != NULL &&
+			    g_slist_find (forb, panel) != NULL)
+				return FALSE;
+		}
+	}
+
+	/* always prefer copy, except for internal icons/applets,
+	 * where we prefer move */
+	if (info == TARGET_ICON_INTERNAL ||
+	    info == TARGET_APPLET_INTERNAL) {
+		if (context->actions & GDK_ACTION_MOVE) {
+			gdk_drag_status (context, GDK_ACTION_MOVE, time);
+		} else {
+			gdk_drag_status (context, context->suggested_action, time);
+		}
+	} else if (context->actions & GDK_ACTION_COPY) {
+		gdk_drag_status (context, GDK_ACTION_COPY, time);
+	} else {
+		gdk_drag_status (context, context->suggested_action, time);
+	}
+
+	do_highlight (widget, TRUE);
 
 	if (IS_BASEP_WIDGET (widget)) {
-		basep_widget_autoshow(BASEP_WIDGET(widget));
-		basep_widget_queue_autohide(BASEP_WIDGET(widget));
+		basep_widget_autoshow (BASEP_WIDGET (widget));
+		basep_widget_queue_autohide (BASEP_WIDGET (widget));
 	}
 
 	return TRUE;
 }
 
 static gboolean
-drag_drop_cb(GtkWidget	       *widget,
-	     GdkDragContext    *context,
-	     gint               x,
-	     gint               y,
-	     guint              time,
-	     Launcher *launcher)
+drag_drop_cb (GtkWidget	        *widget,
+	      GdkDragContext    *context,
+	      gint               x,
+	      gint               y,
+	      guint              time,
+	      Launcher          *launcher)
 {
 	GdkAtom ret_atom = 0;
 
-	if(!is_this_drop_ok(widget, context, NULL, &ret_atom))
+	if ( ! is_this_drop_ok (widget, context, NULL, &ret_atom))
 		return FALSE;
 
 	gtk_drag_get_data(widget, context,
@@ -1105,16 +1331,16 @@ drag_drop_cb(GtkWidget	       *widget,
 }
 
 static void  
-drag_leave_cb(GtkWidget	       *widget,
-	      GdkDragContext   *context,
-	      guint             time,
-	      Launcher *launcher)
+drag_leave_cb (GtkWidget	*widget,
+	       GdkDragContext   *context,
+	       guint             time,
+	       Launcher         *launcher)
 {
 	do_highlight (widget, FALSE);
 }
 
 static void
-drag_data_recieved_cb (GtkWidget	 *widget,
+drag_data_recieved_cb (GtkWidget	*widget,
 		       GdkDragContext   *context,
 		       gint              x,
 		       gint              y,
@@ -1132,8 +1358,8 @@ drag_data_recieved_cb (GtkWidget	 *widget,
 	/* we use this only to really find out the info, we already
 	   know this is an ok drop site and the info that got passed
 	   to us is bogus (it's always 0 in fact) */
-	if(!is_this_drop_ok(widget, context, &info, NULL)) {
-		gtk_drag_finish(context,FALSE,FALSE,time);
+	if ( ! is_this_drop_ok (widget, context, &info, NULL)) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
 	}
 
@@ -1154,39 +1380,44 @@ drag_data_recieved_cb (GtkWidget	 *widget,
 
 	switch (info) {
 	case TARGET_URL:
-		drop_urilist(panel, pos, (char *)selection_data->data,
-			     IS_FOOBAR_WIDGET(widget) ? FALSE : TRUE);
+		drop_urilist (panel, pos, (char *)selection_data->data,
+			      IS_FOOBAR_WIDGET(widget) ? FALSE : TRUE);
 		break;
 	case TARGET_NETSCAPE_URL:
-		drop_url(panel, pos, (char *)selection_data->data);
+		drop_url (panel, pos, (char *)selection_data->data);
 		break;
 	case TARGET_COLOR:
-		drop_color(panel, pos, (guint16 *)selection_data->data);
+		drop_color (panel, pos, (guint16 *)selection_data->data);
 		break;
 	case TARGET_BGIMAGE:
-		if( ! IS_FOOBAR_WIDGET(widget))
-			drop_bgimage(panel, (char *)selection_data->data);
+		if ( ! IS_FOOBAR_WIDGET(widget))
+			drop_bgimage (panel, (char *)selection_data->data);
 		break;
 	case TARGET_DIRECTORY:
-		drop_menu(panel, pos, (char *)selection_data->data);
+		drop_menu (panel, pos, (char *)selection_data->data);
 		break;
 	case TARGET_APPLET:
-		if(!selection_data->data) {
-			gtk_drag_finish(context,FALSE,FALSE,time);
+		if ( ! selection_data->data) {
+			gtk_drag_finish (context, FALSE, FALSE, time);
 			return;
 		}
-		load_extern_applet((char *)selection_data->data, NULL,
-				   panel, pos, TRUE, FALSE);
+		load_extern_applet ((char *)selection_data->data, NULL,
+				    panel, pos, TRUE, FALSE);
 		break;
 	case TARGET_APPLET_INTERNAL:
-		drop_internal_applet(panel, pos, (char *)selection_data->data);
+		drop_internal_applet (panel, pos, (char *)selection_data->data,
+				      context->action);
+		break;
+	case TARGET_ICON_INTERNAL:
+		drop_internal_icon (panel, pos, (char *)selection_data->data,
+				    context->action);
 		break;
 	default:
-		gtk_drag_finish(context,FALSE,FALSE,time);
+		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
 	}
 
-	gtk_drag_finish(context, TRUE, FALSE, time);
+	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
 static void
@@ -1387,15 +1618,17 @@ panel_setup(GtkWidget *panelw)
 	gtk_drag_dest_set (GTK_WIDGET (panelw),
 			   0, NULL, 0, 0);
 
-	gtk_signal_connect(GTK_OBJECT(panelw), "event",
-			   GTK_SIGNAL_FUNC(panel_event),pd);
+	gtk_signal_connect (GTK_OBJECT (panelw), "event",
+			   GTK_SIGNAL_FUNC (panel_event), pd);
+	gtk_signal_connect (GTK_OBJECT (panel), "event",
+			   GTK_SIGNAL_FUNC (panel_widget_event), panelw);
 	
 	gtk_widget_set_events(panelw,
 			      gtk_widget_get_events(panelw) |
 			      PANEL_EVENT_MASK);
  
-	gtk_signal_connect(GTK_OBJECT(panelw), "destroy",
-			   GTK_SIGNAL_FUNC(panel_destroy),NULL);
+	gtk_signal_connect (GTK_OBJECT (panelw), "destroy",
+			    GTK_SIGNAL_FUNC (panel_destroy), NULL);
 
 
 	if(GTK_WIDGET_REALIZED(GTK_WIDGET(panelw)))
@@ -1420,3 +1653,30 @@ send_state_change(void)
 	}
 }
 
+PanelData *
+panel_data_by_id (int id)
+{
+	GSList *list;
+	for(list = panel_list; list != NULL; list = g_slist_next(list)) {
+		PanelData *pd = list->data;
+		int pd_id = -1;
+
+		if (IS_BASEP_WIDGET (pd->panel))
+		       pd_id = PANEL_WIDGET (BASEP_WIDGET (pd->panel)->panel)->unique_id;
+		else if (IS_FOOBAR_WIDGET (pd->panel))
+		       pd_id = PANEL_WIDGET (FOOBAR_WIDGET (pd->panel)->panel)->unique_id;
+
+		if (id == pd_id)
+			return pd;
+	}
+	return NULL;
+}
+
+void
+panel_set_id (GtkWidget *widget, int id)
+{
+	if (IS_BASEP_WIDGET (widget))
+		PANEL_WIDGET (BASEP_WIDGET (widget)->panel)->unique_id = id;
+	else if (IS_FOOBAR_WIDGET (widget))
+		PANEL_WIDGET (FOOBAR_WIDGET (widget)->panel)->unique_id = id;
+}
